@@ -1,15 +1,21 @@
+from collections.abc import Mapping, Sequence
 from enum import Enum
-from typing import Mapping, NamedTuple, Sequence, Union
+from typing import Union
 
 import dagster._check as check
+from dagster._config.config_type import ConfigTypeKind
+from dagster._config.field_utils import EnvVar, IntEnvVar
+from dagster._config.snap import ConfigFieldSnap, ConfigTypeSnap, minimal_config_for_type_snap
+from dagster._config.stack import (
+    EvaluationStackEntry,
+    get_friendly_path_info,
+    get_friendly_path_msg,
+)
+from dagster._config.traversal_context import ContextData
+from dagster._config.type_printer import print_config_type_key_to_string
+from dagster._record import IHaveNew, record, record_custom
 from dagster._utils.error import SerializableErrorInfo
 
-from .config_type import ConfigTypeKind
-from .snap import ConfigFieldSnap, ConfigTypeSnap, minimal_config_for_type_snap
-from .stack import EvaluationStack, get_friendly_path_info, get_friendly_path_msg
-from .traversal_context import ContextData
-from .type_printer import print_config_type_key_to_string
-
 
 class DagsterEvaluationErrorReason(Enum):
     RUNTIME_TYPE_MISMATCH = "RUNTIME_TYPE_MISMATCH"
@@ -22,79 +28,57 @@ class DagsterEvaluationErrorReason(Enum):
     FIELD_ALIAS_COLLISION = "FIELD_ALIAS_COLLISION"
 
 
-class FieldsNotDefinedErrorData(
-    NamedTuple("_FieldsNotDefinedErrorData", [("field_names", Sequence[str])])
-):
-    def __new__(cls, field_names: Sequence[str]):
-        return super(FieldsNotDefinedErrorData, cls).__new__(
-            cls, check.sequence_param(field_names, "field_names", of_type=str)
-        )
+@record
+class FieldsNotDefinedErrorData:
+    field_names: Sequence[str]
 
 
-class FieldAliasCollisionErrorData(NamedTuple):
+@record
+class FieldAliasCollisionErrorData:
     field_name: str
     aliased_field_name: str
 
 
-class FieldNotDefinedErrorData(NamedTuple("_FieldNotDefinedErrorData", [("field_name", str)])):
-    def __new__(cls, field_name: str):
-        return super(FieldNotDefinedErrorData, cls).__new__(
-            cls, check.str_param(field_name, "field_name")
-        )
+@record
+class FieldNotDefinedErrorData:
+    field_name: str
 
 
-class MissingFieldErrorData(
-    NamedTuple("_MissingFieldErrorData", [("field_name", str), ("field_snap", ConfigFieldSnap)])
-):
-    def __new__(cls, field_name, field_snap):
-        return super(MissingFieldErrorData, cls).__new__(
-            cls,
-            check.str_param(field_name, "field_name"),
-            check.inst_param(field_snap, "field_snap", ConfigFieldSnap),
-        )
+@record
+class MissingFieldErrorData:
+    field_name: str
+    field_snap: ConfigFieldSnap
 
 
-class MissingFieldsErrorData(
-    NamedTuple(
-        "_MissingFieldErrorData",
-        [("field_names", Sequence[str]), ("field_snaps", Sequence[ConfigFieldSnap])],
-    )
-):
-    def __new__(cls, field_names, field_snaps):
-        return super(MissingFieldsErrorData, cls).__new__(
-            cls,
-            check.list_param(field_names, "field_names", of_type=str),
-            check.list_param(field_snaps, "field_snaps", of_type=ConfigFieldSnap),
-        )
+@record
+class MissingFieldsErrorData:
+    field_names: Sequence[str]
+    field_snaps: Sequence[ConfigFieldSnap]
 
 
-class RuntimeMismatchErrorData(
-    NamedTuple(
-        "_RuntimeMismatchErrorData", [("config_type_snap", ConfigTypeSnap), ("value_rep", str)]
-    )
-):
-    def __new__(cls, config_type_snap, value_rep):
-        check.inst_param(config_type_snap, "config_type", ConfigTypeSnap)
-        return super(RuntimeMismatchErrorData, cls).__new__(
-            cls,
-            config_type_snap,
-            check.str_param(value_rep, "value_rep"),
-        )
+@record
+class RuntimeMismatchErrorData:
+    config_type_snap: ConfigTypeSnap
+    value_rep: str
 
 
-class SelectorTypeErrorData(
-    NamedTuple(
-        "_SelectorTypeErrorData",
-        [("config_type_snap", ConfigTypeSnap), ("incoming_fields", Sequence[str])],
-    )
-):
-    def __new__(cls, config_type_snap, incoming_fields):
+@record_custom
+class SelectorTypeErrorData(IHaveNew):
+    config_type_snap: ConfigTypeSnap
+    incoming_fields: Sequence[str]
+
+    def __new__(
+        cls,
+        *,
+        config_type_snap: ConfigTypeSnap,
+        incoming_fields: Sequence[str],
+    ):
         check.inst_param(config_type_snap, "config_type_snap", ConfigTypeSnap)
         check.param_invariant(config_type_snap.kind == ConfigTypeKind.SELECTOR, "config_type")
-        return super(SelectorTypeErrorData, cls).__new__(
+        return super().__new__(
             cls,
-            config_type_snap,
-            check.list_param(incoming_fields, "incoming_fields", of_type=str),
+            config_type_snap=config_type_snap,
+            incoming_fields=incoming_fields,
         )
 
 
@@ -112,31 +96,12 @@ ERROR_DATA_UNION = Union[
 ERROR_DATA_TYPES = ERROR_DATA_UNION.__args__  # type: ignore
 
 
-class EvaluationError(
-    NamedTuple(
-        "_EvaluationError",
-        [
-            ("stack", EvaluationStack),
-            ("reason", DagsterEvaluationErrorReason),
-            ("message", str),
-            ("error_data", ERROR_DATA_UNION),
-        ],
-    )
-):
-    def __new__(
-        cls,
-        stack: EvaluationStack,
-        reason: DagsterEvaluationErrorReason,
-        message: str,
-        error_data: ERROR_DATA_UNION,
-    ):
-        return super(EvaluationError, cls).__new__(
-            cls,
-            check.inst_param(stack, "stack", EvaluationStack),
-            check.inst_param(reason, "reason", DagsterEvaluationErrorReason),
-            check.str_param(message, "message"),
-            check.inst_param(error_data, "error_data", ERROR_DATA_TYPES),
-        )
+@record
+class EvaluationError:
+    stack: EvaluationStackEntry
+    reason: DagsterEvaluationErrorReason
+    message: str
+    error_data: ERROR_DATA_UNION
 
 
 def _get_type_msg(type_in_context):
@@ -196,12 +161,8 @@ def create_fields_not_defined_error(
         stack=context.stack,
         reason=DagsterEvaluationErrorReason.FIELDS_NOT_DEFINED,
         message=(
-            'Received unexpected config entries "{undefined_fields}" {path_msg}. '
-            'Expected: "{available_fields}."'
-        ).format(
-            undefined_fields=undefined_fields,
-            path_msg=get_friendly_path_msg(context.stack),
-            available_fields=available_fields,
+            f'Received unexpected config entries "{undefined_fields}" {get_friendly_path_msg(context.stack)}. '
+            f'Expected: "{available_fields}."'
         ),
         error_data=FieldsNotDefinedErrorData(field_names=undefined_fields),
     )
@@ -213,11 +174,11 @@ def create_enum_type_mismatch_error(context: ContextData, config_value: object)
     return EvaluationError(
         stack=context.stack,
         reason=DagsterEvaluationErrorReason.RUNTIME_TYPE_MISMATCH,
-        message="Value {path_msg} for enum type {type_name} must be a string".format(
-            type_name=context.config_type_snap.given_name,
-            path_msg=get_friendly_path_msg(context.stack),
+        message=f"Value {get_friendly_path_msg(context.stack)} for enum type {context.config_type_snap.given_name} must be a string",
+        error_data=RuntimeMismatchErrorData(
+            config_type_snap=context.config_type_snap,
+            value_rep=repr(config_value),
         ),
-        error_data=RuntimeMismatchErrorData(context.config_type_snap, repr(config_value)),
     )
 
 
@@ -227,12 +188,11 @@ def create_enum_value_missing_error(context: ContextData, config_value: object)
     return EvaluationError(
         stack=context.stack,
         reason=DagsterEvaluationErrorReason.RUNTIME_TYPE_MISMATCH,
-        message="Value {path_msg} not in enum type {type_name} got {config_value}".format(
-            config_value=config_value,
-            type_name=context.config_type_snap.given_name,
-            path_msg=get_friendly_path_msg(context.stack),
+        message=f"Value {get_friendly_path_msg(context.stack)} not in enum type {context.config_type_snap.given_name} got {config_value}",
+        error_data=RuntimeMismatchErrorData(
+            config_type_snap=context.config_type_snap,
+            value_rep=repr(config_value),
         ),
-        error_data=RuntimeMismatchErrorData(context.config_type_snap, repr(config_value)),
     )
 
 
@@ -249,8 +209,7 @@ def create_field_not_defined_error(context: ContextData, received_field: str) ->
         stack=context.stack,
         reason=DagsterEvaluationErrorReason.FIELD_NOT_DEFINED,
         message=(
-            'Received unexpected config entry "{received}" {path_msg}. Expected: "{type_name}".'
-            .format(
+            'Received unexpected config entry "{received}" {path_msg}. Expected: "{type_name}".'.format(
                 path_msg=get_friendly_path_msg(context.stack),
                 type_name=print_config_type_key_to_string(
                     context.config_schema_snapshot, context.config_type_key, with_lines=False
@@ -275,7 +234,10 @@ def create_array_error(context: ContextData, config_value: object) -> Evaluation
                 context.config_schema_snapshot, context.config_type_key, with_lines=False
             ),
         ),
-        error_data=RuntimeMismatchErrorData(context.config_type_snap, repr(config_value)),
+        error_data=RuntimeMismatchErrorData(
+            config_type_snap=context.config_type_snap,
+            value_rep=repr(config_value),
+        ),
     )
 
 
@@ -292,7 +254,10 @@ def create_map_error(context: ContextData, config_value: object) -> EvaluationEr
                 context.config_schema_snapshot, context.config_type_key, with_lines=False
             ),
         ),
-        error_data=RuntimeMismatchErrorData(context.config_type_snap, repr(config_value)),
+        error_data=RuntimeMismatchErrorData(
+            config_type_snap=context.config_type_snap,
+            value_rep=repr(config_value),
+        ),
     )
 
 
@@ -366,15 +331,36 @@ def create_scalar_error(context: ContextData, config_value: object) -> Evaluatio
         stack=context.stack,
         reason=DagsterEvaluationErrorReason.RUNTIME_TYPE_MISMATCH,
         message=(
-            'Invalid scalar {path_msg}. Value "{config_value}" of type '
-            '"{type}" is not valid for expected type "{type_name}".'.format(
-                path_msg=get_friendly_path_msg(context.stack),
-                type_name=context.config_type_snap.given_name,
-                config_value=config_value,
-                type=type(config_value),
-            )
+            f'Invalid scalar {get_friendly_path_msg(context.stack)}. Value "{config_value}" of type '
+            f'"{type(config_value)}" is not valid for expected type "{context.config_type_snap.given_name}".'
+        ),
+        error_data=RuntimeMismatchErrorData(
+            config_type_snap=context.config_type_snap,
+            value_rep=repr(config_value),
+        ),
+    )
+
+
+def create_pydantic_env_var_error(
+    context: ContextData, config_value: Union[EnvVar, IntEnvVar]
+) -> EvaluationError:
+    env_var_name = config_value.env_var_name
+
+    correct_env_var = str({"env": env_var_name})
+    return EvaluationError(
+        stack=context.stack,
+        reason=DagsterEvaluationErrorReason.RUNTIME_TYPE_MISMATCH,
+        message=(
+            f'Invalid use of environment variable wrapper. Value "{env_var_name}" is wrapped with'
+            " EnvVar(), which is reserved for passing to structured pydantic config objects only."
+            " To provide an environment variable to a run config dictionary, replace"
+            f' EnvVar("{env_var_name}") with {correct_env_var}, or pass a structured RunConfig'
+            " object."
+        ),
+        error_data=RuntimeMismatchErrorData(
+            config_type_snap=context.config_type_snap,
+            value_rep=repr(env_var_name),
         ),
-        error_data=RuntimeMismatchErrorData(context.config_type_snap, repr(config_value)),
     )
 
 
@@ -390,12 +376,8 @@ def create_selector_multiple_fields_error(
         stack=context.stack,
         reason=DagsterEvaluationErrorReason.SELECTOR_FIELD_ERROR,
         message=(
-            "You can only specify a single field {path_msg}. You specified {incoming_fields}. "
-            "The available fields are {defined_fields}"
-        ).format(
-            incoming_fields=incoming_fields,
-            defined_fields=defined_fields,
-            path_msg=get_friendly_path_msg(context.stack),
+            f"You can only specify a single field {get_friendly_path_msg(context.stack)}. You specified {incoming_fields}. "
+            f"The available fields are {defined_fields}"
         ),
         error_data=SelectorTypeErrorData(
             config_type_snap=context.config_type_snap, incoming_fields=incoming_fields
@@ -414,9 +396,9 @@ def create_selector_multiple_fields_no_field_selected_error(
         stack=context.stack,
         reason=DagsterEvaluationErrorReason.SELECTOR_FIELD_ERROR,
         message=(
-            "Must specify a field {path_msg} if more than one field is defined. "
-            "Defined fields: {defined_fields}"
-        ).format(defined_fields=defined_fields, path_msg=get_friendly_path_msg(context.stack)),
+            f"Must specify a field {get_friendly_path_msg(context.stack)} if more than one field is defined. "
+            f"Defined fields: {defined_fields}"
+        ),
         error_data=SelectorTypeErrorData(
             config_type_snap=context.config_type_snap, incoming_fields=[]
         ),
@@ -429,9 +411,7 @@ def create_selector_type_error(context: ContextData, config_value: object) -> Ev
     return EvaluationError(
         stack=context.stack,
         reason=DagsterEvaluationErrorReason.RUNTIME_TYPE_MISMATCH,
-        message="Value for selector type {path_msg} must be a dict".format(
-            path_msg=get_friendly_path_msg(context.stack)
-        ),
+        message=f"Value for selector type {get_friendly_path_msg(context.stack)} must be a dict",
         error_data=RuntimeMismatchErrorData(
             config_type_snap=context.config_type_snap, value_rep=repr(config_value)
         ),
@@ -447,8 +427,8 @@ def create_selector_unspecified_value_error(context: ContextData) -> EvaluationE
         stack=context.stack,
         reason=DagsterEvaluationErrorReason.SELECTOR_FIELD_ERROR,
         message=(
-            "Must specify the required field {path_msg}. Defined fields: {defined_fields}"
-        ).format(defined_fields=defined_fields, path_msg=get_friendly_path_msg(context.stack)),
+            f"Must specify the required field {get_friendly_path_msg(context.stack)}. Defined fields: {defined_fields}"
+        ),
         error_data=SelectorTypeErrorData(
             config_type_snap=context.config_type_snap, incoming_fields=[]
         ),
@@ -467,7 +447,10 @@ def create_none_not_allowed_error(context: ContextData) -> EvaluationError:
                 context.config_schema_snapshot, context.config_type_key, with_lines=False
             ),
         ),
-        error_data=RuntimeMismatchErrorData(context.config_type_snap, repr(None)),
+        error_data=RuntimeMismatchErrorData(
+            config_type_snap=context.config_type_snap,
+            value_rep=repr(None),
+        ),
     )
 
 
@@ -481,11 +464,7 @@ def create_failed_post_processing_error(
         stack=context.stack,
         reason=DagsterEvaluationErrorReason.FAILED_POST_PROCESSING,
         message=(
-            "Post processing {path_msg} of original value {original_value} failed:\n{error}".format(
-                path_msg=get_friendly_path_msg(context.stack),
-                original_value=original_value,
-                error=error_data.to_string(),
-            )
+            f"Post processing {get_friendly_path_msg(context.stack)} of original value {original_value} failed:\n{error_data.to_string()}"
         ),
         error_data=error_data,
     )
