@@ -1,9 +1,10 @@
 from typing import NamedTuple, Optional
 
+from dagster_shared.serdes import deserialize_value
+from dagster_shared.serdes.errors import DeserializationError
+
 import dagster._check as check
 from dagster._serdes import whitelist_for_serdes
-from dagster._serdes.errors import DeserializationError
-from dagster._serdes.serdes import deserialize_value
 
 
 @whitelist_for_serdes
@@ -14,7 +15,7 @@ class AssetDetails(NamedTuple("_AssetDetails", [("last_wipe_timestamp", Optional
 
     def __new__(cls, last_wipe_timestamp: Optional[float] = None):
         check.opt_float_param(last_wipe_timestamp, "last_wipe_timestamp")
-        return super(AssetDetails, cls).__new__(cls, last_wipe_timestamp)
+        return super().__new__(cls, last_wipe_timestamp)
 
     @staticmethod
     def from_db_string(db_string):
