@@ -1,21 +1,20 @@
+from collections.abc import Mapping, Sequence
 from functools import lru_cache, update_wrapper
 from inspect import Parameter
-from typing import (
+from typing import (  # noqa: UP035
     TYPE_CHECKING,
+    AbstractSet,
     Any,
     Callable,
-    List,
-    Mapping,
     NamedTuple,
     Optional,
-    Sequence,
-    Set,
     Union,
     cast,
     overload,
 )
 
 import dagster._check as check
+from dagster._annotations import deprecated_param, public
 from dagster._config import UserConfigSchema
 from dagster._core.decorator_utils import (
     format_docstring_for_description,
@@ -25,18 +24,17 @@ from dagster._core.decorator_utils import (
     positional_arg_name_list,
 )
 from dagster._core.definitions.inference import infer_input_props
+from dagster._core.definitions.input import In, InputDefinition
+from dagster._core.definitions.output import Out
+from dagster._core.definitions.policy import RetryPolicy
 from dagster._core.definitions.resource_annotation import get_resource_args
+from dagster._core.definitions.utils import DEFAULT_OUTPUT
 from dagster._core.errors import DagsterInvalidDefinitionError
 from dagster._core.types.dagster_type import DagsterTypeKind
-from dagster._utils.backcompat import canonicalize_backcompat_args
-
-from ..input import In, InputDefinition
-from ..output import Out
-from ..policy import RetryPolicy
-from ..utils import DEFAULT_OUTPUT
+from dagster._utils.warnings import config_argument_warning, normalize_renamed_param
 
 if TYPE_CHECKING:
-    from ..op_definition import OpDefinition
+    from dagster._core.definitions.op_definition import OpDefinition
 
 
 class _Op:
@@ -44,7 +42,7 @@ class _Op:
         self,
         name: Optional[str] = None,
         description: Optional[str] = None,
-        required_resource_keys: Optional[Set[str]] = None,
+        required_resource_keys: Optional[AbstractSet[str]] = None,
         config_schema: Optional[Union[Any, Mapping[str, Any]]] = None,
         tags: Optional[Mapping[str, Any]] = None,
         code_version: Optional[str] = None,
@@ -52,6 +50,7 @@ class _Op:
         retry_policy: Optional[RetryPolicy] = None,
         ins: Optional[Mapping[str, In]] = None,
         out: Optional[Union[Out, Mapping[str, Out]]] = None,
+        pool: Optional[str] = None,
     ):
         self.name = check.opt_str_param(name, "name")
         self.decorator_takes_context = check.bool_param(
@@ -60,20 +59,24 @@ class _Op:
 
         self.description = check.opt_str_param(description, "description")
 
-        # these will be checked within SolidDefinition
+        # these will be checked within OpDefinition
         self.required_resource_keys = required_resource_keys
         self.tags = tags
         self.code_version = code_version
         self.retry_policy = retry_policy
+        self.pool = pool
 
-        # config will be checked within SolidDefinition
+        # config will be checked within OpDefinition
         self.config_schema = config_schema
 
         self.ins = check.opt_nullable_mapping_param(ins, "ins", key_type=str, value_type=In)
         self.out = out
 
     def __call__(self, fn: Callable[..., Any]) -> "OpDefinition":
-        from ..op_definition import OpDefinition
+        from dagster._config.pythonic_config import validate_resource_annotated_function
+        from dagster._core.definitions.op_definition import OpDefinition
+
+        validate_resource_annotated_function(fn)
 
         if not self.name:
             self.name = fn.__name__
@@ -84,13 +87,15 @@ class _Op:
             else NoContextDecoratedOpFunction(decorated_fn=fn)
         )
 
+        compute_fn.validate_malformed_config()
+
         if compute_fn.has_config_arg():
             check.param_invariant(
                 self.config_schema is None or self.config_schema == {},
                 "If the @op has a config arg, you cannot specify a config schema",
             )
 
-            from dagster._config.structured_config import infer_schema_from_config_annotation
+            from dagster._config.pythonic_config import infer_schema_from_config_annotation
 
             # Parse schema from the type annotation of the config arg
             config_arg = compute_fn.get_config_arg()
@@ -110,14 +115,12 @@ class _Op:
         decorator_resource_keys = set(self.required_resource_keys or [])
         check.param_invariant(
             len(decorator_resource_keys) == 0 or len(arg_resource_keys) == 0,
-            (
-                "Cannot specify resource requirements in both @op decorator and as arguments to the"
-                " decorated function"
-            ),
+            "Cannot specify resource requirements in both @op decorator and as arguments to the"
+            " decorated function",
         )
         resolved_resource_keys = decorator_resource_keys.union(arg_resource_keys)
 
-        op_def = OpDefinition(
+        op_def = OpDefinition.dagster_internal_init(
             name=self.name,
             ins=self.ins,
             outs=outs,
@@ -128,14 +131,15 @@ class _Op:
             tags=self.tags,
             code_version=self.code_version,
             retry_policy=self.retry_policy,
+            version=None,  # code_version has replaced version
+            pool=self.pool,
         )
         update_wrapper(op_def, compute_fn.decorated_fn)
         return op_def
 
 
 @overload
-def op(compute_fn: Callable[..., Any]) -> "OpDefinition":
-    ...
+def op(compute_fn: Callable[..., Any]) -> "OpDefinition": ...
 
 
 @overload
@@ -146,15 +150,19 @@ def op(
     ins: Optional[Mapping[str, In]] = ...,
     out: Optional[Union[Out, Mapping[str, Out]]] = ...,
     config_schema: Optional[UserConfigSchema] = ...,
-    required_resource_keys: Optional[Set[str]] = ...,
+    required_resource_keys: Optional[AbstractSet[str]] = ...,
     tags: Optional[Mapping[str, Any]] = ...,
     version: Optional[str] = ...,
     retry_policy: Optional[RetryPolicy] = ...,
     code_version: Optional[str] = ...,
-) -> _Op:
-    ...
+    pool: Optional[str] = None,
+) -> _Op: ...
 
 
+@public
+@deprecated_param(
+    param="version", breaking_version="2.0", additional_warn_text="Use `code_version` instead"
+)
 def op(
     compute_fn: Optional[Callable] = None,
     *,
@@ -163,11 +171,12 @@ def op(
     ins: Optional[Mapping[str, In]] = None,
     out: Optional[Union[Out, Mapping[str, Out]]] = None,
     config_schema: Optional[UserConfigSchema] = None,
-    required_resource_keys: Optional[Set[str]] = None,
+    required_resource_keys: Optional[AbstractSet[str]] = None,
     tags: Optional[Mapping[str, Any]] = None,
     version: Optional[str] = None,
     retry_policy: Optional[RetryPolicy] = None,
     code_version: Optional[str] = None,
+    pool: Optional[str] = None,
 ) -> Union["OpDefinition", _Op]:
     """Create an op with the specified parameters from the decorated function.
 
@@ -207,7 +216,7 @@ def op(
         tags (Optional[Dict[str, Any]]): Arbitrary metadata for the op. Frameworks may
             expect and require certain metadata to be attached to a op. Values that are not strings
             will be json encoded and must meet the criteria that `json.loads(json.dumps(value)) == value`.
-        code_version (Optional[str]): (Experimental) Version of the logic encapsulated by the op. If set,
+        code_version (Optional[str]): Version of the logic encapsulated by the op. If set,
             this is used as a default version for all outputs.
         retry_policy (Optional[RetryPolicy]): The retry policy for this op.
 
@@ -235,8 +244,11 @@ def op(
             def multi_out() -> Tuple[str, int]:
                 return 'cool', 4
     """
-    code_version = canonicalize_backcompat_args(
-        code_version, "code_version", version, "version", "2.0"
+    code_version = normalize_renamed_param(
+        code_version,
+        "code_version",
+        version,
+        "version",
     )
 
     if compute_fn is not None:
@@ -258,18 +270,28 @@ def op(
         retry_policy=retry_policy,
         ins=ins,
         out=out,
+        pool=pool,
     )
 
 
 class DecoratedOpFunction(NamedTuple):
-    """Wrapper around the decorated solid function to provide commonly used util methods."""
+    """Wrapper around the decorated op function to provide commonly used util methods."""
 
     decorated_fn: Callable[..., Any]
 
+    @property
+    def name(self):
+        return self.decorated_fn.__name__
+
     @lru_cache(maxsize=1)
     def has_context_arg(self) -> bool:
         return is_context_provided(get_function_params(self.decorated_fn))
 
+    def get_context_arg(self) -> Parameter:
+        if self.has_context_arg():
+            return get_function_params(self.decorated_fn)[0]
+        check.failed("Requested context arg on function that does not have one")
+
     @lru_cache(maxsize=1)
     def _get_function_params(self) -> Sequence[Parameter]:
         return get_function_params(self.decorated_fn)
@@ -281,6 +303,15 @@ class DecoratedOpFunction(NamedTuple):
 
         return False
 
+    def validate_malformed_config(self) -> None:
+        from dagster._config.pythonic_config.config import Config
+        from dagster._config.pythonic_config.type_check_utils import safe_is_subclass
+
+        positional_inputs = self.positional_inputs()
+        for param in get_function_params(self.decorated_fn):
+            if safe_is_subclass(param.annotation, Config) and param.name in positional_inputs:
+                config_argument_warning(param.name, self.name)
+
     def get_config_arg(self) -> Parameter:
         for param in get_function_params(self.decorated_fn):
             if param.name == "config":
@@ -308,14 +339,14 @@ class DecoratedOpFunction(NamedTuple):
         return len(params) > 0 and param_is_var_keyword(params[-1])
 
     def get_output_annotation(self) -> Any:
-        from ..inference import infer_output_props
+        from dagster._core.definitions.inference import infer_output_props
 
         return infer_output_props(self.decorated_fn).annotation
 
 
 class NoContextDecoratedOpFunction(DecoratedOpFunction):
-    """Wrapper around a decorated solid function, when the decorator does not permit a context
-    parameter (such as lambda_solid).
+    """Wrapper around a decorated op function, when the decorator does not permit a context
+    parameter.
     """
 
     @lru_cache(maxsize=1)
@@ -329,7 +360,7 @@ def is_context_provided(params: Sequence[Parameter]) -> bool:
     return params[0].name in get_valid_name_permutations("context")
 
 
-def resolve_checked_solid_fn_inputs(
+def resolve_checked_op_fn_inputs(
     decorator_name: str,
     fn_name: str,
     compute_fn: DecoratedOpFunction,
@@ -340,10 +371,10 @@ def resolve_checked_solid_fn_inputs(
     Returns the resolved set of InputDefinitions.
 
     Args:
-        decorator_name (str): Name of the decorator that is wrapping the op/solid function.
+        decorator_name (str): Name of the decorator that is wrapping the op function.
         fn_name (str): Name of the decorated function.
-        compute_fn (DecoratedSolidFunction): The decorated function, wrapped in the
-            DecoratedSolidFunction wrapper.
+        compute_fn (DecoratedOpFunction): The decorated function, wrapped in the
+            DecoratedOpFunction wrapper.
         explicit_input_defs (List[InputDefinition]): The input definitions that were explicitly
             provided in the decorator.
         exclude_nothing (bool): True if Nothing type inputs should be excluded from compute_fn
@@ -385,7 +416,7 @@ def resolve_checked_solid_fn_inputs(
     inputs_to_infer = set()
     has_kwargs = False
 
-    for param in cast(List[Parameter], input_args):
+    for param in cast("list[Parameter]", input_args):
         if param.kind == Parameter.VAR_KEYWORD:
             has_kwargs = True
         elif param.kind == Parameter.VAR_POSITIONAL:
@@ -413,11 +444,16 @@ def resolve_checked_solid_fn_inputs(
     undeclared_inputs = explicit_names - used_inputs
     if not has_kwargs and undeclared_inputs:
         undeclared_inputs_printed = ", '".join(undeclared_inputs)
+        nothing_exemption = (
+            ", except for Ins that have the Nothing dagster_type"
+            if decorator_name not in {"@graph", "@graph_asset"}
+            else ""
+        )
         raise DagsterInvalidDefinitionError(
             f"{decorator_name} '{fn_name}' decorated function does not have argument(s)"
-            f" '{undeclared_inputs_printed}'. {decorator_name}-decorated functions should have a"
-            " keyword argument for each of their Ins, except for Ins that have the Nothing"
-            " dagster_type. Alternatively, they can accept **kwargs."
+            f" '{undeclared_inputs_printed}'. {decorator_name}-decorated functions should have an"
+            f" argument for each of their Ins{nothing_exemption}. Alternatively, they can"
+            " accept **kwargs."
         )
 
     inferred_props = {
