@@ -1,29 +1,24 @@
 import inspect
-from typing import (
-    TYPE_CHECKING,
-    AbstractSet,
-    Any,
-    Callable,
-    Iterator,
-    List,
-    Mapping,
-    Optional,
-    Sequence,
-    Tuple,
-    TypeVar,
-    Union,
-    cast,
-)
+from collections.abc import Iterator, Mapping, Sequence, Set
+from typing import TYPE_CHECKING, AbstractSet, Any, Callable, Optional, Union, cast  # noqa: UP035
 
 from typing_extensions import TypeAlias, get_args, get_origin
 
 import dagster._check as check
-from dagster._annotations import public
+from dagster._annotations import deprecated, deprecated_param, public
 from dagster._config.config_schema import UserConfigSchema
-from dagster._core.decorator_utils import get_function_params
-from dagster._core.definitions.dependency import NodeHandle, NodeInputHandle
+from dagster._core.definitions.asset_checks.asset_check_result import AssetCheckResult
+from dagster._core.definitions.definition_config_schema import (
+    IDefinitionConfigSchema,
+    convert_user_facing_definition_config_schema,
+)
+from dagster._core.definitions.dependency import NodeHandle, NodeInputHandle, NodeOutputHandle
+from dagster._core.definitions.hook_definition import HookDefinition
+from dagster._core.definitions.inference import infer_output_props
+from dagster._core.definitions.input import In, InputDefinition
 from dagster._core.definitions.node_definition import NodeDefinition
-from dagster._core.definitions.op_invocation import op_invocation_result
+from dagster._core.definitions.op_invocation import direct_invocation_result
+from dagster._core.definitions.output import Out, OutputDefinition
 from dagster._core.definitions.policy import RetryPolicy
 from dagster._core.definitions.resource_requirement import (
     InputManagerRequirement,
@@ -31,34 +26,35 @@ from dagster._core.definitions.resource_requirement import (
     OutputManagerRequirement,
     ResourceRequirement,
 )
-from dagster._core.errors import DagsterInvalidInvocationError, DagsterInvariantViolationError
-from dagster._core.types.dagster_type import DagsterType, DagsterTypeKind
-from dagster._utils.backcompat import canonicalize_backcompat_args, deprecation_warning
-
-from .definition_config_schema import (
-    IDefinitionConfigSchema,
-    convert_user_facing_definition_config_schema,
+from dagster._core.definitions.result import MaterializeResult, ObserveResult
+from dagster._core.definitions.utils import DEFAULT_IO_MANAGER_KEY, check_valid_chars
+from dagster._core.errors import (
+    DagsterInvalidDefinitionError,
+    DagsterInvalidInvocationError,
+    DagsterInvariantViolationError,
 )
-from .hook_definition import HookDefinition
-from .inference import infer_output_props
-from .input import In, InputDefinition
-from .output import Out, OutputDefinition
+from dagster._core.storage.tags import GLOBAL_CONCURRENCY_TAG
+from dagster._core.types.dagster_type import DagsterType, DagsterTypeKind
+from dagster._utils import IHasInternalInit
+from dagster._utils.warnings import normalize_renamed_param
 
 if TYPE_CHECKING:
-    from dagster._core.definitions.asset_layer import AssetLayer
-
-    from .composition import PendingNodeInvocation
-    from .decorators.op_decorator import DecoratedOpFunction
+    from dagster._core.definitions.assets.job.asset_layer import AssetLayer
+    from dagster._core.definitions.composition import PendingNodeInvocation
+    from dagster._core.definitions.decorators.op_decorator import DecoratedOpFunction
 
 OpComputeFunction: TypeAlias = Callable[..., Any]
 
 
-class OpDefinition(NodeDefinition):
+@public
+@deprecated_param(
+    param="version",
+    breaking_version="2.0",
+    additional_warn_text="Use `code_version` instead.",
+)
+class OpDefinition(NodeDefinition, IHasInternalInit):
     """Defines an op, the functional unit of user-defined computation.
 
-    For more details on what a op is, refer to the
-    `Ops Overview <../../concepts/ops-jobs-graphs/ops>`_ .
-
     End users should prefer the :func:`@op <op>` decorator. OpDefinition is generally intended to be
     used by framework authors or for programatically generated ops.
 
@@ -85,9 +81,10 @@ class OpDefinition(NodeDefinition):
             not set metadata directly. Values that are not strings will be json encoded and must meet
             the criteria that `json.loads(json.dumps(value)) == value`.
         required_resource_keys (Optional[Set[str]]): Set of resources handles required by this op.
-        code_version (Optional[str]): (Experimental) Version of the code encapsulated by the op. If set,
+        code_version (Optional[str]): Version of the code encapsulated by the op. If set,
             this is used as a default code version for all outputs.
         retry_policy (Optional[RetryPolicy]): The retry policy for this op.
+        pool (Optional[str]): A string that identifies the pool that governs this op's execution.
 
 
     Examples:
@@ -109,6 +106,7 @@ class OpDefinition(NodeDefinition):
     _required_resource_keys: AbstractSet[str]
     _version: Optional[str]
     _retry_policy: Optional[RetryPolicy]
+    _pool: Optional[str]
 
     def __init__(
         self,
@@ -123,8 +121,12 @@ class OpDefinition(NodeDefinition):
         version: Optional[str] = None,
         retry_policy: Optional[RetryPolicy] = None,
         code_version: Optional[str] = None,
+        pool: Optional[str] = None,
     ):
-        from .decorators.op_decorator import DecoratedOpFunction, resolve_checked_solid_fn_inputs
+        from dagster._core.definitions.decorators.op_decorator import (
+            DecoratedOpFunction,
+            resolve_checked_op_fn_inputs,
+        )
 
         ins = check.opt_mapping_param(ins, "ins")
         input_defs = [
@@ -132,20 +134,25 @@ class OpDefinition(NodeDefinition):
         ]  # sort so that input definition order is deterministic
 
         if isinstance(compute_fn, DecoratedOpFunction):
-            resolved_input_defs: Sequence[InputDefinition] = resolve_checked_solid_fn_inputs(
+            resolved_input_defs: Sequence[InputDefinition] = resolve_checked_op_fn_inputs(
                 decorator_name="@op",
                 fn_name=name,
-                compute_fn=cast(DecoratedOpFunction, compute_fn),
+                compute_fn=cast("DecoratedOpFunction", compute_fn),
                 explicit_input_defs=input_defs,
                 exclude_nothing=True,
             )
             self._compute_fn = compute_fn
+            _validate_context_type_hint(self._compute_fn.decorated_fn)
         else:
             resolved_input_defs = input_defs
             self._compute_fn = check.callable_param(compute_fn, "compute_fn")
+            _validate_context_type_hint(self._compute_fn)
 
-        code_version = canonicalize_backcompat_args(
-            code_version, "code_version", version, "version", "2.0"
+        code_version = normalize_renamed_param(
+            code_version,
+            "code_version",
+            version,
+            "version",
         )
         self._version = code_version
 
@@ -159,6 +166,8 @@ class OpDefinition(NodeDefinition):
             check.opt_set_param(required_resource_keys, "required_resource_keys", of_type=str)
         )
         self._retry_policy = check.opt_inst_param(retry_policy, "retry_policy", RetryPolicy)
+        self._pool = pool
+        pool = _validate_pool(pool, tags)
 
         positional_inputs = (
             self._compute_fn.positional_inputs()
@@ -166,15 +175,45 @@ class OpDefinition(NodeDefinition):
             else None
         )
 
-        super(OpDefinition, self).__init__(
+        super().__init__(
             name=name,
             input_defs=check.sequence_param(resolved_input_defs, "input_defs", InputDefinition),
             output_defs=check.sequence_param(output_defs, "output_defs", OutputDefinition),
             description=description,
-            tags=check.opt_mapping_param(tags, "tags", key_type=str),
+            tags=(check.opt_mapping_param(tags, "tags", key_type=str)),
             positional_inputs=positional_inputs,
         )
 
+    def dagster_internal_init(
+        *,
+        compute_fn: Union[Callable[..., Any], "DecoratedOpFunction"],
+        name: str,
+        ins: Optional[Mapping[str, In]],
+        outs: Optional[Mapping[str, Out]],
+        description: Optional[str],
+        config_schema: Optional[Union[UserConfigSchema, IDefinitionConfigSchema]],
+        required_resource_keys: Optional[AbstractSet[str]],
+        tags: Optional[Mapping[str, Any]],
+        version: Optional[str],
+        retry_policy: Optional[RetryPolicy],
+        code_version: Optional[str],
+        pool: Optional[str],
+    ) -> "OpDefinition":
+        return OpDefinition(
+            compute_fn=compute_fn,
+            name=name,
+            ins=ins,
+            outs=outs,
+            description=description,
+            config_schema=config_schema,
+            required_resource_keys=required_resource_keys,
+            tags=tags,
+            version=version,
+            retry_policy=retry_policy,
+            code_version=code_version,
+            pool=pool,
+        )
+
     @property
     def node_type_str(self) -> str:
         return "op"
@@ -186,16 +225,19 @@ class OpDefinition(NodeDefinition):
     @public
     @property
     def name(self) -> str:
-        return super(OpDefinition, self).name
+        """str: The name of this op."""
+        return super().name
 
     @public
     @property
     def ins(self) -> Mapping[str, In]:
+        """Mapping[str, In]: A mapping from input name to the In object that represents that input."""
         return {input_def.name: In.from_definition(input_def) for input_def in self.input_defs}
 
     @public
     @property
     def outs(self) -> Mapping[str, Out]:
+        """Mapping[str, Out]: A mapping from output name to the Out object that represents that output."""
         return {output_def.name: Out.from_definition(output_def) for output_def in self.output_defs}
 
     @property
@@ -205,47 +247,68 @@ class OpDefinition(NodeDefinition):
     @public
     @property
     def config_schema(self) -> IDefinitionConfigSchema:
+        """IDefinitionConfigSchema: The config schema for this op."""
         return self._config_schema
 
     @public
     @property
     def required_resource_keys(self) -> AbstractSet[str]:
+        """AbstractSet[str]: A set of keys for resources that must be provided to this OpDefinition."""
         return frozenset(self._required_resource_keys)
 
     @public
+    @deprecated(breaking_version="2.0", additional_warn_text="Use `code_version` instead.")
     @property
     def version(self) -> Optional[str]:
-        deprecation_warning("`version` property on OpDefinition", "2.0")
+        """str: Version of the code encapsulated by the op. If set, this is used as a
+        default code version for all outputs.
+        """
         return self._version
 
     @public
     @property
     def retry_policy(self) -> Optional[RetryPolicy]:
+        """Optional[RetryPolicy]: The RetryPolicy for this op."""
         return self._retry_policy
 
     @public
     @property
     def tags(self) -> Mapping[str, str]:
-        return super(OpDefinition, self).tags
+        """Mapping[str, str]: The tags for this op."""
+        return super().tags
 
     @public
     def alias(self, name: str) -> "PendingNodeInvocation":
-        return super(OpDefinition, self).alias(name)
+        """Creates a copy of this op with the given name."""
+        return super().alias(name)
 
     @public
     def tag(self, tags: Optional[Mapping[str, str]]) -> "PendingNodeInvocation":
-        return super(OpDefinition, self).tag(tags)
+        """Creates a copy of this op with the given tags."""
+        return super().tag(tags)
 
     @public
     def with_hooks(self, hook_defs: AbstractSet[HookDefinition]) -> "PendingNodeInvocation":
-        return super(OpDefinition, self).with_hooks(hook_defs)
+        """Creates a copy of this op with the given hook definitions."""
+        return super().with_hooks(hook_defs)
 
     @public
     def with_retry_policy(self, retry_policy: RetryPolicy) -> "PendingNodeInvocation":
-        return super(OpDefinition, self).with_retry_policy(retry_policy)
+        """Creates a copy of this op with the given retry policy."""
+        return super().with_retry_policy(retry_policy)
+
+    @property
+    def pool(self) -> Optional[str]:
+        """Optional[str]: The concurrency pool for this op."""
+        return self._pool
+
+    @property
+    def pools(self) -> Set[str]:
+        """Optional[str]: The concurrency pools for this op node."""
+        return {self._pool} if self._pool else set()
 
     def is_from_decorator(self) -> bool:
-        from .decorators.op_decorator import DecoratedOpFunction
+        from dagster._core.definitions.decorators.op_decorator import DecoratedOpFunction
 
         return isinstance(self._compute_fn, DecoratedOpFunction)
 
@@ -266,11 +329,9 @@ class OpDefinition(NodeDefinition):
     def iterate_op_defs(self) -> Iterator["OpDefinition"]:
         yield self
 
-    T_Handle = TypeVar("T_Handle", bound=Optional[NodeHandle])
-
     def resolve_output_to_origin(
-        self, output_name: str, handle: T_Handle
-    ) -> Tuple[OutputDefinition, T_Handle]:
+        self, output_name: str, handle: Optional[NodeHandle]
+    ) -> tuple[OutputDefinition, Optional[NodeHandle]]:
         return self.output_def_named(output_name), handle
 
     def resolve_output_to_origin_op_def(self, output_name: str) -> "OpDefinition":
@@ -279,17 +340,16 @@ class OpDefinition(NodeDefinition):
     def get_inputs_must_be_resolved_top_level(
         self, asset_layer: "AssetLayer", handle: Optional[NodeHandle] = None
     ) -> Sequence[InputDefinition]:
-        handle = cast(NodeHandle, check.inst_param(handle, "handle", NodeHandle))
+        handle = cast("NodeHandle", check.inst_param(handle, "handle", NodeHandle))
         unresolveable_input_defs = []
         for input_def in self.input_defs:
             if (
                 not input_def.dagster_type.loader
                 and not input_def.dagster_type.kind == DagsterTypeKind.NOTHING
-                and not input_def.root_manager_key
                 and not input_def.has_default_value
                 and not input_def.input_manager_key
             ):
-                input_asset_key = asset_layer.asset_key_for_input(handle, input_def.name)
+                input_asset_key = asset_layer.get_asset_key_for_node_input(handle, input_def.name)
                 # If input_asset_key is present, this input can be resolved
                 # by a source asset, so input does not need to be resolved
                 # at the top level.
@@ -307,52 +367,59 @@ class OpDefinition(NodeDefinition):
     def input_supports_dynamic_output_dep(self, input_name: str) -> bool:
         return True
 
+    def with_replaced_properties(
+        self,
+        name: str,
+        ins: Optional[Mapping[str, In]] = None,
+        outs: Optional[Mapping[str, Out]] = None,
+        config_schema: Optional[IDefinitionConfigSchema] = None,
+        description: Optional[str] = None,
+    ) -> "OpDefinition":
+        return OpDefinition.dagster_internal_init(
+            name=name,
+            ins={input_def.name: In.from_definition(input_def) for input_def in self.input_defs}
+            if ins is None
+            else ins,
+            outs={
+                output_def.name: Out.from_definition(output_def) for output_def in self.output_defs
+            }
+            if outs is None
+            else outs,
+            compute_fn=self.compute_fn,
+            config_schema=config_schema or self.config_schema,
+            description=description or self.description,
+            tags=self.tags,
+            required_resource_keys=self.required_resource_keys,
+            code_version=self._version,
+            retry_policy=self.retry_policy,
+            version=None,  # code_version replaces version
+            pool=self.pool,
+        )
+
     def copy_for_configured(
         self,
         name: str,
         description: Optional[str],
         config_schema: IDefinitionConfigSchema,
     ) -> "OpDefinition":
-        return OpDefinition(
+        return self.with_replaced_properties(
             name=name,
-            ins={input_def.name: In.from_definition(input_def) for input_def in self.input_defs},
-            outs={
-                output_def.name: Out.from_definition(output_def) for output_def in self.output_defs
-            },
-            compute_fn=self.compute_fn,
+            description=description,
             config_schema=config_schema,
-            description=description or self.description,
-            tags=self.tags,
-            required_resource_keys=self.required_resource_keys,
-            code_version=self.version,
-            retry_policy=self.retry_policy,
         )
 
     def get_resource_requirements(
         self,
-        outer_context: Optional[object] = None,
+        handle: Optional[NodeHandle],
+        asset_layer: Optional["AssetLayer"],
     ) -> Iterator[ResourceRequirement]:
-        # Outer requiree in this context is the outer-calling node handle. If not provided, then just use the solid name.
-        outer_context = cast(Optional[Tuple[NodeHandle, Optional["AssetLayer"]]], outer_context)
-        if not outer_context:
-            handle = None
-            asset_layer = None
-        else:
-            handle, asset_layer = outer_context
         node_description = f"{self.node_type_str} '{handle or self.name}'"
         for resource_key in sorted(list(self.required_resource_keys)):
             yield OpDefinitionResourceRequirement(
                 key=resource_key, node_description=node_description
             )
         for input_def in self.input_defs:
-            if input_def.root_manager_key:
-                yield InputManagerRequirement(
-                    key=input_def.root_manager_key,
-                    node_description=node_description,
-                    input_name=input_def.name,
-                    root_input=True,
-                )
-            elif input_def.input_manager_key:
+            if input_def.input_manager_key:
                 yield InputManagerRequirement(
                     key=input_def.input_manager_key,
                     node_description=node_description,
@@ -360,9 +427,13 @@ class OpDefinition(NodeDefinition):
                     root_input=False,
                 )
             elif asset_layer and handle:
-                input_asset_key = asset_layer.asset_key_for_input(handle, input_def.name)
+                input_asset_key = asset_layer.get_asset_key_for_node_input(handle, input_def.name)
                 if input_asset_key:
-                    io_manager_key = asset_layer.io_manager_key_for_asset(input_asset_key)
+                    io_manager_key = (
+                        asset_layer.get(input_asset_key).io_manager_key
+                        if asset_layer.has(input_asset_key)
+                        else DEFAULT_IO_MANAGER_KEY
+                    )
                     yield InputManagerRequirement(
                         key=io_manager_key,
                         node_description=node_description,
@@ -382,59 +453,27 @@ class OpDefinition(NodeDefinition):
     ) -> Sequence[NodeInputHandle]:
         return [input_handle]
 
+    def resolve_output_to_destinations(
+        self, output_name: str, handle: Optional[NodeHandle]
+    ) -> Sequence[NodeInputHandle]:
+        return []
+
     def __call__(self, *args, **kwargs) -> Any:
-        from ..execution.context.invocation import UnboundOpExecutionContext
-        from .composition import is_in_composition
-        from .decorators.op_decorator import DecoratedOpFunction
+        from dagster._core.definitions.composition import is_in_composition
 
         if is_in_composition():
-            return super(OpDefinition, self).__call__(*args, **kwargs)
-        else:
-            node_label = self.node_type_str  # string "solid" for solids, "op" for ops
+            return super().__call__(*args, **kwargs)
 
-            if not isinstance(self.compute_fn, DecoratedOpFunction):
-                raise DagsterInvalidInvocationError(
-                    f"Attemped to invoke {node_label} that was not constructed using the"
-                    f" `@{node_label}` decorator. Only {node_label}s constructed using the"
-                    f" `@{node_label}` decorator can be directly invoked."
-                )
-            if self.compute_fn.has_context_arg():
-                if len(args) + len(kwargs) == 0:
-                    raise DagsterInvalidInvocationError(
-                        f"Compute function of {node_label} '{self.name}' has context argument, but"
-                        " no context was provided when invoking."
-                    )
-                if len(args) > 0:
-                    if args[0] is not None and not isinstance(args[0], UnboundOpExecutionContext):
-                        raise DagsterInvalidInvocationError(
-                            f"Compute function of {node_label} '{self.name}' has context argument, "
-                            "but no context was provided when invoking."
-                        )
-                    context = args[0]
-                    return op_invocation_result(self, context, *args[1:], **kwargs)
-                # Context argument is provided under kwargs
-                else:
-                    context_param_name = get_function_params(self.compute_fn.decorated_fn)[0].name
-                    if context_param_name not in kwargs:
-                        raise DagsterInvalidInvocationError(
-                            f"Compute function of {node_label} '{self.name}' has context argument "
-                            f"'{context_param_name}', but no value for '{context_param_name}' was "
-                            f"found when invoking. Provided kwargs: {kwargs}"
-                        )
-                    context = cast(UnboundOpExecutionContext, kwargs[context_param_name])
-                    kwargs_sans_context = {
-                        kwarg: val
-                        for kwarg, val in kwargs.items()
-                        if not kwarg == context_param_name
-                    }
-                    return op_invocation_result(self, context, *args, **kwargs_sans_context)
+        return direct_invocation_result(self, *args, **kwargs)
 
-            else:
-                context = None
-                if len(args) > 0 and isinstance(args[0], UnboundOpExecutionContext):
-                    context = cast(UnboundOpExecutionContext, args[0])
-                    args = args[1:]
-                return op_invocation_result(self, context, *args, **kwargs)
+    def get_op_handles(self, parent: NodeHandle) -> AbstractSet[NodeHandle]:
+        return {parent}
+
+    def get_op_output_handles(self, parent: Optional[NodeHandle]) -> AbstractSet[NodeOutputHandle]:
+        return {
+            NodeOutputHandle(node_handle=parent, output_name=output_def.name)
+            for output_def in self.output_defs
+        }
 
 
 def _resolve_output_defs_from_outs(
@@ -442,7 +481,7 @@ def _resolve_output_defs_from_outs(
     outs: Optional[Mapping[str, Out]],
     default_code_version: Optional[str],
 ) -> Sequence[OutputDefinition]:
-    from .decorators.op_decorator import DecoratedOpFunction
+    from dagster._core.definitions.decorators.op_decorator import DecoratedOpFunction
 
     if isinstance(compute_fn, DecoratedOpFunction):
         inferred_output_props = infer_output_props(compute_fn.decorated_fn)
@@ -459,36 +498,122 @@ def _resolve_output_defs_from_outs(
     # If only a single entry has been provided to the out dict, then slurp the
     # annotation into the entry.
     if len(outs) == 1:
-        name = list(outs.keys())[0]
+        name = next(iter(outs.keys()))
         only_out = outs[name]
         return [only_out.to_definition(annotation, name, description, default_code_version)]
 
-    output_defs: List[OutputDefinition] = []
+    # If multiple outputs...
 
-    # Introspection on type annotations is experimental, so checking
-    # metaclass is the best we can do.
-    if annotation != inspect.Parameter.empty and not get_origin(annotation) == tuple:
+    # Note: we don't provide description when using multiple outputs. Introspection
+    # is challenging when faced with multiple outputs.
+
+    # ... and no annotation, use empty for each output annotation
+    if annotation == inspect.Parameter.empty:
+        return [
+            out.to_definition(
+                annotation_type=inspect.Parameter.empty,
+                name=name,
+                description=None,
+                code_version=default_code_version,
+            )
+            for (name, out) in outs.items()
+        ]
+
+    # ... or if a single result object type, use None for each output annotation
+    if _is_result_object_type(annotation):
+        # this can happen for example when there are outputs for checks
+        # that get reported via a singular MaterializeResult
+        return [
+            out.to_definition(
+                annotation_type=type(None),
+                name=name,
+                description=None,
+                code_version=default_code_version,
+            )
+            for (name, out) in outs.items()
+        ]
+
+    # ... otherwise we expect to have a tuple with entries...
+    if get_origin(annotation) != tuple:
         raise DagsterInvariantViolationError(
             "Expected Tuple annotation for multiple outputs, but received non-tuple annotation."
         )
-    if annotation != inspect.Parameter.empty and not len(get_args(annotation)) == len(outs):
+    subtypes = get_args(annotation)
+
+    # ... if they are all result object entries use None
+    if len(subtypes) > 0 and all(_is_result_object_type(t) for t in subtypes):
+        # the counts of subtypes and outputs may not align due to checks results
+        # being passed via MaterializeResult similar to above.
+        return [
+            out.to_definition(
+                annotation_type=type(None),
+                name=name,
+                description=None,
+                code_version=default_code_version,
+            )
+            for (name, out) in outs.items()
+        ]
+
+    # ... otherwise they should align with outputs
+    if len(subtypes) != len(outs):
         raise DagsterInvariantViolationError(
             "Expected Tuple annotation to have number of entries matching the "
             f"number of outputs for more than one output. Expected {len(outs)} "
-            f"outputs but annotation has {len(get_args(annotation))}."
+            f"outputs but annotation has {len(subtypes)}."
         )
-    for idx, (name, cur_out) in enumerate(outs.items()):
-        annotation_type = (
-            get_args(annotation)[idx]
-            if annotation != inspect.Parameter.empty
-            else inspect.Parameter.empty
+    return [
+        cur_out.to_definition(
+            annotation_type=subtypes[idx],
+            name=name,
+            description=None,
+            code_version=default_code_version,
         )
-        # Don't provide description when using multiple outputs. Introspection
-        # is challenging when faced with multiple inputs.
-        output_defs.append(
-            cur_out.to_definition(
-                annotation_type, name=name, description=None, code_version=default_code_version
+        for idx, (name, cur_out) in enumerate(outs.items())
+    ]
+
+
+def _validate_context_type_hint(fn):
+    from inspect import _empty as EmptyAnnotation
+
+    from dagster._core.decorator_utils import get_function_params
+    from dagster._core.definitions.decorators.op_decorator import is_context_provided
+    from dagster._core.execution.context.compute import (
+        AssetCheckExecutionContext,
+        AssetExecutionContext,
+        OpExecutionContext,
+    )
+
+    params = get_function_params(fn)
+    if is_context_provided(params):
+        if params[0].annotation not in [
+            AssetExecutionContext,
+            OpExecutionContext,
+            EmptyAnnotation,
+            AssetCheckExecutionContext,
+        ]:
+            raise DagsterInvalidDefinitionError(
+                f"Cannot annotate `context` parameter with type {params[0].annotation}. `context`"
+                " must be annotated with AssetExecutionContext, AssetCheckExecutionContext, OpExecutionContext, or left blank."
             )
+
+
+def _is_result_object_type(ttype):
+    # Is this type special result object type
+    return ttype in (MaterializeResult, ObserveResult, AssetCheckResult)
+
+
+def _validate_pool(pool, tags):
+    check.opt_str_param(pool, "pool")
+    if not pool:
+        return None
+
+    check_valid_chars(pool)
+
+    tags = check.opt_mapping_param(tags, "tags")
+    tag_concurrency_key = tags.get(GLOBAL_CONCURRENCY_TAG)
+    if tag_concurrency_key and pool != tag_concurrency_key:
+        raise DagsterInvalidDefinitionError(
+            f'Pool "{pool}" conflicts with the concurrency key tag "{tag_concurrency_key}".'
         )
 
-    return output_defs
+    return pool
