@@ -1,31 +1,25 @@
 import inspect
-from typing import (
-    Any,
-    NamedTuple,
-    Optional,
-    Type,
-    TypeVar,
-    Union,
-)
+from typing import Any, NamedTuple, Optional, TypeVar, Union
+
+from dagster_shared.error import DagsterError
 
 import dagster._check as check
-from dagster._annotations import PublicAttr
+from dagster._annotations import PublicAttr, deprecated_param, public
+from dagster._core.definitions.inference import InferredOutputProps
+from dagster._core.definitions.input import NoValueSentinel
 from dagster._core.definitions.metadata import (
     ArbitraryMetadataMapping,
-    MetadataUserInput,
+    RawMetadataMapping,
     normalize_metadata,
 )
-from dagster._core.errors import DagsterError, DagsterInvalidDefinitionError
+from dagster._core.definitions.utils import DEFAULT_IO_MANAGER_KEY, DEFAULT_OUTPUT, check_valid_name
+from dagster._core.errors import DagsterInvalidDefinitionError
 from dagster._core.types.dagster_type import (
     DagsterType,
     is_dynamic_output_annotation,
     resolve_dagster_type,
 )
 
-from .inference import InferredOutputProps
-from .input import NoValueSentinel
-from .utils import DEFAULT_IO_MANAGER_KEY, DEFAULT_OUTPUT, check_valid_name
-
 TOutputDefinition = TypeVar("TOutputDefinition", bound="OutputDefinition")
 TOut = TypeVar("TOut", bound="Out")
 
@@ -54,7 +48,7 @@ class OutputDefinition:
             For example, users can provide a file path if the data object will be stored in a
             filesystem, or provide information of a database table when it is going to load the data
             into the table.
-        code_version (Optional[str]): (Experimental) Version of the code that generates this output. In
+        code_version (Optional[str]): Version of the code that generates this output. In
             general, versions should be set only for code that deterministically produces the same
             output when given the same inputs.
 
@@ -126,7 +120,7 @@ class OutputDefinition:
     ) -> "OutputMapping":
         """Create an output mapping from an output of a child node.
 
-        In a CompositeSolidDefinition, you can use this helper function to construct
+        In a GraphDefinition, you can use this helper function to construct
         an :py:class:`OutputMapping` from the output of a child node.
 
         Args:
@@ -205,12 +199,13 @@ def _checked_inferred_type(inferred: Any) -> DagsterType:
         ) from e
 
 
+@public
 class DynamicOutputDefinition(OutputDefinition):
     """Variant of :py:class:`OutputDefinition <dagster.OutputDefinition>` for an
     output that will dynamically alter the graph at runtime.
 
-    When using in a composition function such as :py:func:`@pipeline <dagster.pipeline>`,
-    dynamic outputs must be used with either
+    When using in a composition function such as :py:func:`@job <dagster.job>`,
+    dynamic outputs must be used with either:
 
     * ``map`` - clone downstream nodes for each separate :py:class:`DynamicOutput`
     * ``collect`` - gather across all :py:class:`DynamicOutput` in to a list
@@ -231,7 +226,7 @@ class DynamicOutputDefinition(OutputDefinition):
                 for file in filenames:
                     yield DynamicOutput(os.path.join(dirname, file), mapping_key=_clean(file))
 
-            @pipeline
+            @job
             def process_directory():
                 files = files_in_directory()
 
@@ -249,13 +244,22 @@ class DynamicOutputDefinition(OutputDefinition):
 
 class OutputPointer(NamedTuple("_OutputPointer", [("node_name", str), ("output_name", str)])):
     def __new__(cls, node_name: str, output_name: Optional[str] = None):
-        return super(OutputPointer, cls).__new__(
+        return super().__new__(
             cls,
             check.str_param(node_name, "node_name"),
             check.opt_str_param(output_name, "output_name", DEFAULT_OUTPUT),
         )
 
 
+@public
+@deprecated_param(
+    param="dagster_type",
+    breaking_version="2.0",
+    additional_warn_text="Any defined `dagster_type` should come from the underlying op `Output`.",
+    # Disabling warning here since we're passing this internally and I'm not sure whether it is
+    # actually used or discarded.
+    emit_runtime_warning=False,
+)
 class OutputMapping(NamedTuple):
     """Defines an output mapping for a graph.
 
@@ -265,7 +269,7 @@ class OutputMapping(NamedTuple):
         mapped_node_output_name (str): Name of the output in the node (op/graph) that is being mapped from.
         graph_output_description (Optional[str]): A description of the output in the graph being mapped from.
         from_dynamic_mapping (bool): Set to true if the node being mapped to is a mapped dynamic node.
-        dagster_type (Optional[DagsterType]): (Deprecated) The dagster type of the graph's output being mapped to. Users should not use this argument when instantiating the class.
+        dagster_type (Optional[DagsterType]): The dagster type of the graph's output being mapped to.
 
     Examples:
         .. code-block:: python
@@ -316,15 +320,16 @@ class OutputMapping(NamedTuple):
         )
 
 
+@public
 class Out(
     NamedTuple(
         "_Out",
         [
-            ("dagster_type", PublicAttr[Union[DagsterType, Type[NoValueSentinel]]]),
+            ("dagster_type", PublicAttr[Union[DagsterType, type[NoValueSentinel]]]),
             ("description", PublicAttr[Optional[str]]),
             ("is_required", PublicAttr[bool]),
             ("io_manager_key", PublicAttr[str]),
-            ("metadata", PublicAttr[Optional[MetadataUserInput]]),
+            ("metadata", PublicAttr[Optional[RawMetadataMapping]]),
             ("code_version", PublicAttr[Optional[str]]),
         ],
     )
@@ -350,14 +355,14 @@ class Out(
             For example, users can provide a file path if the data object will be stored in a
             filesystem, or provide information of a database table when it is going to load the data
             into the table.
-        code_version (Optional[str]): (Experimental) Version of the code that generates this output. In
+        code_version (Optional[str]): Version of the code that generates this output. In
             general, versions should be set only for code that deterministically produces the same
             output when given the same inputs.
     """
 
     def __new__(
         cls,
-        dagster_type: Union[Type, DagsterType] = NoValueSentinel,
+        dagster_type: Optional[Union[type, DagsterType]] = NoValueSentinel,
         description: Optional[str] = None,
         is_required: bool = True,
         io_manager_key: Optional[str] = None,
@@ -365,11 +370,13 @@ class Out(
         code_version: Optional[str] = None,
         # make sure new parameters are updated in combine_with_inferred below
     ):
-        return super(Out, cls).__new__(
+        return super().__new__(
             cls,
-            dagster_type=NoValueSentinel
-            if dagster_type is NoValueSentinel
-            else resolve_dagster_type(dagster_type),
+            dagster_type=(
+                NoValueSentinel
+                if dagster_type is NoValueSentinel
+                else resolve_dagster_type(dagster_type)
+            ),
             description=description,
             is_required=check.bool_param(is_required, "is_required"),
             io_manager_key=check.opt_str_param(
@@ -421,6 +428,7 @@ class Out(
         return False
 
 
+@public
 class DynamicOut(Out):
     """Variant of :py:class:`Out <dagster.Out>` for an output that will dynamically alter the graph at
     runtime.
@@ -486,6 +494,7 @@ class DynamicOut(Out):
         return True
 
 
+@public
 class GraphOut(NamedTuple("_GraphOut", [("description", PublicAttr[Optional[str]])])):
     """Represents information about the outputs that a graph maps.
 
@@ -494,7 +503,7 @@ class GraphOut(NamedTuple("_GraphOut", [("description", PublicAttr[Optional[str]
     """
 
     def __new__(cls, description: Optional[str] = None):
-        return super(GraphOut, cls).__new__(cls, description=description)
+        return super().__new__(cls, description=description)
 
     def to_definition(self, name: Optional[str]) -> "OutputDefinition":
         return OutputDefinition(name=name, description=self.description)
