@@ -1,14 +1,15 @@
-from .repository_definition import (
-    AssetsDefinitionCacheableData as AssetsDefinitionCacheableData,
+from dagster._core.definitions.repository_definition.repository_data import (
     CachingRepositoryData as CachingRepositoryData,
-    PendingRepositoryDefinition as PendingRepositoryDefinition,
     RepositoryData as RepositoryData,
+)
+from dagster._core.definitions.repository_definition.repository_definition import (
+    AssetsDefinitionCacheableData as AssetsDefinitionCacheableData,
     RepositoryDefinition as RepositoryDefinition,
     RepositoryLoadData as RepositoryLoadData,
 )
-from .valid_definitions import (
+from dagster._core.definitions.repository_definition.valid_definitions import (
     SINGLETON_REPOSITORY_NAME as SINGLETON_REPOSITORY_NAME,
     VALID_REPOSITORY_DATA_DICT_KEYS as VALID_REPOSITORY_DATA_DICT_KEYS,
-    PendingRepositoryListDefinition as PendingRepositoryListDefinition,
-    RepositoryListDefinition as RepositoryListDefinition,
+    RepositoryElementDefinition as RepositoryElementDefinition,
+    RepositoryListSpec as RepositoryListSpec,
 )
