@@ -1,19 +1,7 @@
 import time
+from collections.abc import Iterator, Mapping, Sequence
 from types import TracebackType
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    Iterator,
-    List,
-    Mapping,
-    Optional,
-    Sequence,
-    Set,
-    Type,
-    Union,
-    cast,
-)
+from typing import Any, Callable, Optional, Union, cast
 
 from typing_extensions import Self
 
@@ -29,21 +17,30 @@ from dagster._core.execution.context.system import (
     PlanExecutionContext,
     PlanOrchestrationContext,
 )
+from dagster._core.execution.plan.instance_concurrency_context import InstanceConcurrencyContext
+from dagster._core.execution.plan.outputs import StepOutputData, StepOutputHandle
+from dagster._core.execution.plan.plan import ExecutionPlan
 from dagster._core.execution.plan.state import KnownExecutionState
+from dagster._core.execution.plan.step import ExecutionStep
 from dagster._core.execution.retries import RetryMode, RetryState
-from dagster._core.storage.tags import PRIORITY_TAG
+from dagster._core.storage.tags import GLOBAL_CONCURRENCY_TAG, PRIORITY_TAG
 from dagster._utils.interrupts import pop_captured_interrupt
 from dagster._utils.tags import TagConcurrencyLimitsCounter
 
-from .outputs import StepOutputData, StepOutputHandle
-from .plan import ExecutionPlan
-from .step import ExecutionStep
-
 
 def _default_sort_key(step: ExecutionStep) -> float:
     return int(step.tags.get(PRIORITY_TAG, 0)) * -1
 
 
+def _pool_key_for_step(step: ExecutionStep) -> Optional[str]:
+    # for backwards compatibility, we also check the tags
+    return step.pool or step.tags.get(GLOBAL_CONCURRENCY_TAG)
+
+
+CONCURRENCY_CLAIM_BLOCKED_INTERVAL = 1
+CONCURRENCY_CLAIM_MESSAGE_INTERVAL = 300
+
+
 class ActiveExecution:
     """State machine used to track progress through execution of an ExecutionPlan."""
 
@@ -53,13 +50,15 @@ class ActiveExecution:
         retry_mode: RetryMode,
         sort_key_fn: Optional[Callable[[ExecutionStep], float]] = None,
         max_concurrent: Optional[int] = None,
-        tag_concurrency_limits: Optional[List[Dict[str, Any]]] = None,
+        tag_concurrency_limits: Optional[list[dict[str, Any]]] = None,
+        instance_concurrency_context: Optional[InstanceConcurrencyContext] = None,
     ):
         self._plan: ExecutionPlan = check.inst_param(
             execution_plan, "execution_plan", ExecutionPlan
         )
         self._retry_mode = check.inst_param(retry_mode, "retry_mode", RetryMode)
         self._retry_state = self._plan.known_state.get_retry_state()
+        self._instance_concurrency_context = instance_concurrency_context
 
         self._sort_key_fn: Callable[[ExecutionStep], float] = (
             check.opt_callable_param(
@@ -77,41 +76,42 @@ class ActiveExecution:
         self._context_guard: bool = False  # Prevent accidental direct use
 
         # We decide what steps to skip based on what outputs are yielded by upstream steps
-        self._step_outputs: Set[StepOutputHandle] = set(self._plan.known_state.ready_outputs)
+        self._step_outputs: set[StepOutputHandle] = set(self._plan.known_state.ready_outputs)
 
         # All steps to be executed start out here in _pending
-        self._pending: Dict[str, Set[str]] = dict(self._plan.get_executable_step_deps())
+        self._pending: dict[str, set[str]] = dict(self._plan.get_executable_step_deps())
 
         # track mapping keys from DynamicOutputs, step_key, output_name -> list of keys
         # to _gathering while in flight
-        self._gathering_dynamic_outputs: Dict[str, Mapping[str, Optional[List[str]]]] = {}
+        self._gathering_dynamic_outputs: dict[str, Mapping[str, Optional[list[str]]]] = {}
         # then on resolution move to _completed
-        self._completed_dynamic_outputs: Dict[str, Mapping[str, Optional[Sequence[str]]]] = (
+        self._completed_dynamic_outputs: dict[str, Mapping[str, Optional[Sequence[str]]]] = (
             dict(self._plan.known_state.dynamic_mappings) if self._plan.known_state else {}
         )
         self._new_dynamic_mappings: bool = False
 
         # track which upstream deps caused a step to skip
-        self._skipped_deps: Dict[str, Sequence[str]] = {}
+        self._skipped_deps: dict[str, Sequence[str]] = {}
 
         # steps move in to these buckets as a result of _update calls
-        self._executable: List[str] = []
-        self._pending_skip: List[str] = []
-        self._pending_retry: List[str] = []
-        self._pending_abandon: List[str] = []
-        self._waiting_to_retry: Dict[str, float] = {}
+        self._executable: list[str] = []
+        self._pending_skip: list[str] = []
+        self._pending_retry: list[str] = []
+        self._pending_abandon: list[str] = []
+        self._waiting_to_retry: dict[str, float] = {}
+        self._messaged_concurrency_slots: dict[str, float] = {}
 
         # then are considered _in_flight when vended via get_steps_to_*
-        self._in_flight: Set[str] = set()
+        self._in_flight: set[str] = set()
 
         # and finally their terminal state is tracked by these sets, via mark_*
-        self._success: Set[str] = set()
-        self._failed: Set[str] = set()
-        self._skipped: Set[str] = set()
-        self._abandoned: Set[str] = set()
+        self._success: set[str] = set()
+        self._failed: set[str] = set()
+        self._skipped: set[str] = set()
+        self._abandoned: set[str] = set()
 
         # see verify_complete
-        self._unknown_state: Set[str] = set()
+        self._unknown_state: set[str] = set()
 
         self._interrupted: bool = False
 
@@ -123,7 +123,10 @@ class ActiveExecution:
         return self
 
     def __exit__(
-        self, exc_type: Type[Exception], exc_value: Exception, traceback: TracebackType
+        self,
+        exc_type: Optional[type[BaseException]],
+        exc_value: Optional[BaseException],
+        traceback: Optional[TracebackType],
     ) -> None:
         self._context_guard = False
 
@@ -132,21 +135,12 @@ class ActiveExecution:
             return
 
         if not self.is_complete:
-            pending_action = (
-                self._executable + self._pending_abandon + self._pending_retry + self._pending_skip
-            )
-            state_str = "{pending_str}{in_flight_str}{action_str}{retry_str}".format(
-                in_flight_str=f"\nSteps still in flight: {self._in_flight}"
-                if self._in_flight
-                else "",
-                pending_str=f"\nSteps pending processing: {self._pending.keys()}"
-                if self._pending
-                else "",
-                action_str=f"\nSteps pending action: {pending_action}" if pending_action else "",
-                retry_str=f"\nSteps waiting to retry: {self._waiting_to_retry.keys()}"
-                if self._waiting_to_retry
-                else "",
-            )
+            # generate the state string before exiting the concurrency context
+            state_str = self._pending_state_str()
+        else:
+            state_str = ""
+
+        if not self.is_complete:
             if self._interrupted:
                 raise DagsterExecutionInterruptedError(
                     f"Execution was interrupted before completing the execution plan. {state_str}"
@@ -161,26 +155,74 @@ class ActiveExecution:
         if len(self._unknown_state) > 0:
             if self._interrupted:
                 raise DagsterExecutionInterruptedError(
-                    "Execution exited with steps {step_list} in an unknown state after "
-                    "being interrupted.".format(step_list=self._unknown_state)
+                    f"Execution exited with steps {self._unknown_state} in an unknown state after "
+                    "being interrupted."
                 )
             else:
                 raise DagsterUnknownStepStateError(
-                    "Execution exited with steps {step_list} in an unknown state to this"
+                    f"Execution exited with steps {self._unknown_state} in an unknown state to this"
                     " process.\nThis was likely caused by losing communication with the process"
-                    " performing step execution.".format(step_list=self._unknown_state)
+                    " performing step execution."
                 )
 
+    def _pending_state_str(self) -> str:
+        assert not self.is_complete
+        pending_action = (
+            self._executable + self._pending_abandon + self._pending_retry + self._pending_skip
+        )
+        return "{pending_str}{in_flight_str}{action_str}{retry_str}{claim_str}".format(
+            in_flight_str=f"\nSteps still in flight: {self._in_flight}" if self._in_flight else "",
+            pending_str=(
+                f"\nSteps pending processing: {self._pending.keys()}" if self._pending else ""
+            ),
+            action_str=f"\nSteps pending action: {pending_action}" if pending_action else "",
+            retry_str=(
+                f"\nSteps waiting to retry: {self._waiting_to_retry.keys()}"
+                if self._waiting_to_retry
+                else ""
+            ),
+            claim_str=(
+                "\nSteps waiting to claim:"
+                f" {self._instance_concurrency_context.pending_claim_steps()}"
+                if self._instance_concurrency_context
+                and self._instance_concurrency_context.has_pending_claims()
+                else ""
+            ),
+        )
+
+    def _should_skip_step(self, step_key: str, successful_or_skipped_steps: set[str]) -> bool:
+        step = self.get_step_by_key(step_key)
+        for step_input in step.step_inputs:
+            missing_source_handles = []
+
+            for source_handle in step_input.get_step_output_handle_dependencies():
+                if (
+                    source_handle.step_key in successful_or_skipped_steps
+                    and source_handle not in self._step_outputs
+                ):
+                    missing_source_handles.append(source_handle)
+
+            if missing_source_handles:
+                if len(missing_source_handles) == len(
+                    step_input.get_step_output_handle_dependencies()
+                ):
+                    self._skipped_deps[step_key] = [
+                        f"{h.step_key}.{h.output_name}" for h in missing_source_handles
+                    ]
+                    return True
+        return False
+
     def _update(self) -> None:
         """Moves steps from _pending to _executable / _pending_skip / _pending_retry
         as a function of what has been _completed.
         """
-        new_steps_to_execute: List[str] = []
-        new_steps_to_skip: List[str] = []
-        new_steps_to_abandon: List[str] = []
+        new_steps_to_execute: list[str] = []
+        new_steps_to_skip: list[str] = []
+        new_steps_to_abandon: list[str] = []
 
         successful_or_skipped_steps = self._success | self._skipped
         failed_or_abandoned_steps = self._failed | self._abandoned
+        resolved_steps = self._success | self._skipped | self._failed | self._abandoned
 
         if self._new_dynamic_mappings:
             new_step_deps = self._plan.resolve(self._completed_dynamic_outputs)
@@ -189,39 +231,12 @@ class ActiveExecution:
 
             self._new_dynamic_mappings = False
 
-        for step_key, requirements in self._pending.items():
-            # If any upstream deps failed - this is not executable
-            if requirements.intersection(failed_or_abandoned_steps):
-                new_steps_to_abandon.append(step_key)
-
-            # If all the upstream steps of a step are complete or skipped
-            elif requirements.issubset(successful_or_skipped_steps):
-                step = self.get_step_by_key(step_key)
-
-                # The base case is downstream step won't skip
-                should_skip = False
-
-                # If there is at least one of the step's inputs, none of whose upstream steps has
-                # yielded an output, we should skip that step.
-                for step_input in step.step_inputs:
-                    missing_source_handles = [
-                        source_handle
-                        for source_handle in step_input.get_step_output_handle_dependencies()
-                        if source_handle.step_key in requirements
-                        and source_handle not in self._step_outputs
-                    ]
-                    if missing_source_handles:
-                        if len(missing_source_handles) == len(
-                            step_input.get_step_output_handle_dependencies()
-                        ):
-                            should_skip = True
-                            self._skipped_deps[step_key] = [
-                                f"{h.step_key}.{h.output_name}" for h in missing_source_handles
-                            ]
-                            break
-
-                if should_skip:
+        for step_key, depends_on_steps in self._pending.items():
+            if depends_on_steps.issubset(resolved_steps):
+                if self._should_skip_step(step_key, successful_or_skipped_steps):
                     new_steps_to_skip.append(step_key)
+                elif depends_on_steps.intersection(failed_or_abandoned_steps):
+                    new_steps_to_abandon.append(step_key)
                 else:
                     new_steps_to_execute.append(step_key)
 
@@ -247,30 +262,42 @@ class ActiveExecution:
             self._executable.append(key)
             del self._waiting_to_retry[key]
 
-    def sleep_til_ready(self) -> None:
+    def sleep_interval(self):
         now = time.time()
-        sleep_amt = min([ready_at - now for ready_at in self._waiting_to_retry.values()])
+        intervals = []
+        if self._waiting_to_retry:
+            for t in self._waiting_to_retry.values():
+                intervals.append(t - now)
+        if (
+            self._instance_concurrency_context
+            and self._instance_concurrency_context.has_pending_claims()
+        ):
+            intervals.append(
+                self._instance_concurrency_context.interval_to_next_pending_claim_check()
+            )
+        if intervals:
+            return min(intervals)
+
+        return 0
+
+    def sleep_til_ready(self) -> None:
+        sleep_amt = self.sleep_interval()
         if sleep_amt > 0:
             time.sleep(sleep_amt)
 
-    def get_next_step(self) -> ExecutionStep:
+    def get_next_step(self) -> Optional[ExecutionStep]:
         check.invariant(not self.is_complete, "Can not call get_next_step when is_complete is True")
 
         steps = self.get_steps_to_execute(limit=1)
-        step = None
 
-        if steps:
-            step = steps[0]
-        elif self._waiting_to_retry:
-            self.sleep_til_ready()
-            step = self.get_next_step()
+        if not steps:
+            return None
 
-        check.invariant(step is not None, "Unexpected ActiveExecution state")
-        return step  # type: ignore  # (possible none)
+        return steps[0]
 
     def get_step_by_key(self, step_key: str) -> ExecutionStep:
         step = self._plan.get_step_by_key(step_key)
-        return cast(ExecutionStep, check.inst(step, ExecutionStep))
+        return check.inst(step, ExecutionStep)
 
     def get_steps_to_execute(
         self,
@@ -289,15 +316,15 @@ class ActiveExecution:
             key=self._sort_key_fn,
         )
 
-        tag_concurrency_limits_counter = None
+        run_scoped_concurrency_limits_counter = None
         if self._tag_concurrency_limits:
             in_flight_steps = [self.get_step_by_key(key) for key in self._in_flight]
-            tag_concurrency_limits_counter = TagConcurrencyLimitsCounter(
+            run_scoped_concurrency_limits_counter = TagConcurrencyLimitsCounter(
                 self._tag_concurrency_limits,
                 in_flight_steps,
             )
 
-        batch: List[ExecutionStep] = []
+        batch: list[ExecutionStep] = []
 
         for step in steps:
             if limit is not None and len(batch) >= limit:
@@ -309,11 +336,25 @@ class ActiveExecution:
             ):
                 break
 
-            if tag_concurrency_limits_counter:
-                if tag_concurrency_limits_counter.is_blocked(step):
+            if run_scoped_concurrency_limits_counter:
+                if run_scoped_concurrency_limits_counter.is_blocked(step):
                     continue
 
-                tag_concurrency_limits_counter.update_counters_with_launched_item(step)
+            if run_scoped_concurrency_limits_counter:
+                run_scoped_concurrency_limits_counter.update_counters_with_launched_item(step)
+
+            # fallback to fetching from tags for backwards compatibility
+            concurrency_key = _pool_key_for_step(step)
+            if concurrency_key and self._instance_concurrency_context:
+                try:
+                    step_priority = int(step.tags.get(PRIORITY_TAG, 0))
+                except ValueError:
+                    step_priority = 0
+
+                if not self._instance_concurrency_context.claim(
+                    concurrency_key, step.key, step_priority
+                ):
+                    continue
 
             batch.append(step)
 
@@ -334,7 +375,7 @@ class ActiveExecution:
             steps.append(step)
             self._in_flight.add(key)
             self._pending_skip.remove(key)
-            self._gathering_dynamic_outputs
+            self._gathering_dynamic_outputs  # noqa: B018
             self._skip_for_dynamic_outputs(step)
 
         return sorted(steps, key=self._sort_key_fn)
@@ -352,13 +393,13 @@ class ActiveExecution:
         return sorted(steps, key=self._sort_key_fn)
 
     def plan_events_iterator(
-        self, pipeline_context: Union[PlanExecutionContext, PlanOrchestrationContext]
+        self, job_context: Union[PlanExecutionContext, PlanOrchestrationContext]
     ) -> Iterator[DagsterEvent]:
         """Process all steps that can be skipped and abandoned."""
         steps_to_skip = self.get_steps_to_skip()
         while steps_to_skip:
             for step in steps_to_skip:
-                step_context = pipeline_context.for_step(step)
+                step_context = job_context.for_step(step)
                 step_context.log.info(
                     f"Skipping step {step.key} due to skipped dependencies:"
                     f" {self._skipped_deps[step.key]}."
@@ -372,12 +413,12 @@ class ActiveExecution:
         steps_to_abandon = self.get_steps_to_abandon()
         while steps_to_abandon:
             for step in steps_to_abandon:
-                step_context = pipeline_context.for_step(step)
-                failed_inputs: List[str] = []
+                step_context = job_context.for_step(step)
+                failed_inputs: list[str] = []
                 for step_input in step.step_inputs:
                     failed_inputs.extend(self._failed.intersection(step_input.dependency_keys))
 
-                abandoned_inputs: List[str] = []
+                abandoned_inputs: list[str] = []
                 for step_input in step.step_inputs:
                     abandoned_inputs.extend(
                         self._abandoned.intersection(step_input.dependency_keys)
@@ -387,9 +428,9 @@ class ActiveExecution:
                     "Dependencies for step {step}{fail_str}{abandon_str}. Not executing.".format(
                         step=step.key,
                         fail_str=f" failed: {failed_inputs}" if failed_inputs else "",
-                        abandon_str=f" were not executed: {abandoned_inputs}"
-                        if abandoned_inputs
-                        else "",
+                        abandon_str=(
+                            f" were not executed: {abandoned_inputs}" if abandoned_inputs else ""
+                        ),
                     )
                 )
                 self.mark_abandoned(step.key)
@@ -445,18 +486,18 @@ class ActiveExecution:
     def _mark_complete(self, step_key: str) -> None:
         check.invariant(
             step_key in self._in_flight,
-            "Attempted to mark step {} as complete that was not known to be in flight".format(
-                step_key
-            ),
+            f"Attempted to mark step {step_key} as complete that was not known to be in flight",
         )
         self._in_flight.remove(step_key)
 
     def handle_event(self, dagster_event: DagsterEvent) -> None:
         check.inst_param(dagster_event, "dagster_event", DagsterEvent)
 
-        step_key = cast(str, dagster_event.step_key)
+        step_key = cast("str", dagster_event.step_key)
         if dagster_event.is_step_failure:
             self.mark_failed(step_key)
+            if self._instance_concurrency_context:
+                self._instance_concurrency_context.free_step(step_key)
         elif dagster_event.is_resource_init_failure:
             # Resources are only initialized without a step key in the
             # in-process case, and resource initalization happens before the
@@ -466,8 +507,12 @@ class ActiveExecution:
                 "Resource init failure was reported during execution without a step key.",
             )
             self.mark_failed(step_key)
+            if self._instance_concurrency_context:
+                self._instance_concurrency_context.free_step(step_key)
         elif dagster_event.is_step_success:
             self.mark_success(step_key)
+            if self._instance_concurrency_context:
+                self._instance_concurrency_context.free_step(step_key)
         elif dagster_event.is_step_skipped:
             # Skip events are generated by this class. They should not be sent via handle_event
             raise DagsterInvariantViolationError(
@@ -476,12 +521,16 @@ class ActiveExecution:
         elif dagster_event.is_step_up_for_retry:
             self.mark_up_for_retry(
                 step_key,
-                time.time() + dagster_event.step_retry_data.seconds_to_wait
-                if dagster_event.step_retry_data.seconds_to_wait
-                else None,
+                (
+                    time.time() + dagster_event.step_retry_data.seconds_to_wait
+                    if dagster_event.step_retry_data.seconds_to_wait
+                    else None
+                ),
             )
+            if self._instance_concurrency_context:
+                self._instance_concurrency_context.free_step(step_key)
         elif dagster_event.is_successful_output:
-            event_specific_data = cast(StepOutputData, dagster_event.event_specific_data)
+            event_specific_data = cast("StepOutputData", dagster_event.event_specific_data)
             self.mark_step_produced_output(event_specific_data.step_output_handle)
             if dagster_event.step_output_data.step_output_handle.mapping_key:
                 check.not_none(
@@ -490,13 +539,12 @@ class ActiveExecution:
                     ],
                 ).append(dagster_event.step_output_data.step_output_handle.mapping_key)
 
-    def verify_complete(self, pipeline_context: IPlanContext, step_key: str) -> None:
-        """Ensure that a step has reached a terminal state, if it has not mark it as an unexpected failure.
-        """
+    def verify_complete(self, job_context: IPlanContext, step_key: str) -> None:
+        """Ensure that a step has reached a terminal state, if it has not mark it as an unexpected failure."""
         if step_key in self._in_flight:
-            pipeline_context.log.error(
-                "Step {key} finished without success or failure event. Downstream steps will not"
-                " execute.".format(key=step_key)
+            job_context.log.error(
+                f"Step {step_key} finished without success or failure event. Downstream steps will not"
+                " execute."
             )
             self.mark_unknown_state(step_key)
 
@@ -521,12 +569,20 @@ class ActiveExecution:
             and len(self._pending_retry) == 0
             and len(self._pending_abandon) == 0
             and len(self._waiting_to_retry) == 0
+            and (
+                not self._instance_concurrency_context
+                or not self._instance_concurrency_context.has_pending_claims()
+            )
         )
 
     @property
     def retry_state(self) -> RetryState:
         return self._retry_state
 
+    @property
+    def has_in_flight_steps(self) -> bool:
+        return len(self._in_flight) > 0
+
     def get_known_state(self) -> KnownExecutionState:
         return KnownExecutionState(
             previous_retry_attempts=self._retry_state.snapshot_attempts(),
@@ -550,7 +606,7 @@ class ActiveExecution:
     def _resolve_any_dynamic_outputs(self, step_key: str) -> None:
         if step_key in self._gathering_dynamic_outputs:
             step = self.get_step_by_key(step_key)
-            completed_mappings: Dict[str, Optional[Sequence[str]]] = {}
+            completed_mappings: dict[str, Optional[Sequence[str]]] = {}
             for output_name, mappings in self._gathering_dynamic_outputs[step_key].items():
                 # if no dynamic outputs were returned and the output was marked is_required=False
                 # set to None to indicate a skip should occur
@@ -578,3 +634,25 @@ class ActiveExecution:
             self.get_steps_to_execute()
 
         return [self.get_step_by_key(step_key) for step_key in self._in_flight]
+
+    def concurrency_event_iterator(
+        self, plan_context: Union[PlanExecutionContext, PlanOrchestrationContext]
+    ) -> Iterator[DagsterEvent]:
+        if not self._instance_concurrency_context:
+            return
+
+        pending_claims = self._instance_concurrency_context.pending_claim_steps()
+        for step_key in pending_claims:
+            last_messaged_timestamp = self._messaged_concurrency_slots.get(step_key)
+            if (
+                not last_messaged_timestamp
+                or time.time() - last_messaged_timestamp > CONCURRENCY_CLAIM_MESSAGE_INTERVAL
+            ):
+                step = self.get_step_by_key(step_key)
+                step_context = plan_context.for_step(step)
+                pool = check.inst(_pool_key_for_step(step), str)
+                self._messaged_concurrency_slots[step_key] = time.time()
+                is_initial_message = last_messaged_timestamp is None
+                yield DagsterEvent.step_concurrency_blocked(
+                    step_context, pool, initial=is_initial_message
+                )
