@@ -1,4 +1,4 @@
-from .base import (
+from dagster._core.executor.step_delegating.step_handler.base import (
     CheckStepHealthResult as CheckStepHealthResult,
     StepHandler as StepHandler,
     StepHandlerContext as StepHandlerContext,
