@@ -1,40 +1,31 @@
 import re
 import sys
 from collections import defaultdict, deque
-from typing import (
+from collections.abc import Hashable, Iterable, Mapping, MutableSet, Sequence
+from typing import (  # noqa: UP035
     TYPE_CHECKING,
     AbstractSet,
-    Any,
     Callable,
-    Dict,
-    FrozenSet,
     Generic,
-    Hashable,
-    Iterable,
-    List,
-    Mapping,
-    MutableSet,
     NamedTuple,
     Optional,
-    Sequence,
-    Set,
-    Tuple,
     TypeVar,
-    Union,
 )
 
-from typing_extensions import Final, Literal, TypeAlias
+from typing_extensions import Literal, TypeAlias
 
+from dagster._core.definitions.asset_checks.asset_check_spec import AssetCheckKey
 from dagster._core.definitions.dependency import DependencyStructure
 from dagster._core.definitions.events import AssetKey
 from dagster._core.errors import DagsterExecutionStepNotFoundError, DagsterInvalidSubsetError
+from dagster._record import record
 from dagster._utils import check
 
 if TYPE_CHECKING:
-    from dagster._core.definitions.assets import AssetsDefinition
+    from dagster._core.definitions.assets.definition.assets_definition import AssetsDefinition
+    from dagster._core.definitions.assets.graph.base_asset_graph import BaseAssetGraph, T_AssetNode
+    from dagster._core.definitions.graph_definition import GraphDefinition
     from dagster._core.definitions.job_definition import JobDefinition
-    from dagster._core.definitions.pipeline_definition import PipelineDefinition
-    from dagster._core.definitions.source_asset import SourceAsset
 
 MAX_NUM = sys.maxsize
 
@@ -56,7 +47,7 @@ class OpSelectionData(
 ):
     """The data about op selection.
 
-    Attributes:
+    Args:
         op_selection (List[str]): The queries of op selection.
         resolved_op_selection (AbstractSet[str]): The names of selected ops.
         parent_job_def (JobDefinition): The definition of the full job. This is used for constructing
@@ -71,7 +62,7 @@ class OpSelectionData(
     ):
         from dagster._core.definitions.job_definition import JobDefinition
 
-        return super(OpSelectionData, cls).__new__(
+        return super().__new__(
             cls,
             op_selection=check.sequence_param(op_selection, "op_selection", str),
             resolved_op_selection=check.set_param(
@@ -81,61 +72,42 @@ class OpSelectionData(
         )
 
 
-class AssetSelectionData(
-    NamedTuple(
-        "_AssetSelectionData",
-        [
-            ("asset_selection", AbstractSet[AssetKey]),
-            ("parent_job_def", "JobDefinition"),
-        ],
-    )
-):
+@record
+class AssetSelectionData:
     """The data about asset selection.
 
-    Attributes:
+    Args:
         asset_selection (FrozenSet[AssetKey]): The set of assets to be materialized within the job.
         parent_job_def (JobDefinition): The definition of the full job. This is used for constructing
             pipeline snapshot lineage.
     """
 
-    def __new__(cls, asset_selection: AbstractSet[AssetKey], parent_job_def: "JobDefinition"):
-        from dagster._core.definitions.job_definition import JobDefinition
-
-        return super(AssetSelectionData, cls).__new__(
-            cls,
-            asset_selection=check.set_param(asset_selection, "asset_selection", AssetKey),
-            parent_job_def=check.inst_param(parent_job_def, "parent_job_def", JobDefinition),
-        )
+    asset_selection: AbstractSet[AssetKey]
+    asset_check_selection: Optional[AbstractSet[AssetCheckKey]]
+    parent_job_def: "JobDefinition"
 
 
 def generate_asset_dep_graph(
-    assets_defs: Iterable["AssetsDefinition"], source_assets: Iterable["SourceAsset"]
+    assets_defs: Iterable["AssetsDefinition"],
 ) -> DependencyGraph[AssetKey]:
-    from dagster._core.definitions.resolved_asset_deps import ResolvedAssetDependencies
-
-    resolved_asset_deps = ResolvedAssetDependencies(assets_defs, source_assets)
-
-    upstream: Dict[AssetKey, Set[AssetKey]] = {}
-    downstream: Dict[AssetKey, Set[AssetKey]] = {}
+    upstream: dict[AssetKey, set[AssetKey]] = {}
+    downstream: dict[AssetKey, set[AssetKey]] = {}
     for assets_def in assets_defs:
         for asset_key in assets_def.keys:
             upstream[asset_key] = set()
             downstream[asset_key] = downstream.get(asset_key, set())
             # for each asset upstream of this one, set that as upstream, and this downstream of it
-            upstream_asset_keys = resolved_asset_deps.get_resolved_upstream_asset_keys(
-                assets_def, asset_key
-            )
-            for upstream_key in upstream_asset_keys:
-                upstream[asset_key].add(upstream_key)
-                downstream[upstream_key] = downstream.get(upstream_key, set()) | {asset_key}
+            for dep in assets_def.specs_by_key[asset_key].deps:
+                upstream[asset_key].add(dep.asset_key)
+                downstream[dep.asset_key] = downstream.get(dep.asset_key, set()) | {asset_key}
     return {"upstream": upstream, "downstream": downstream}
 
 
-def generate_dep_graph(pipeline_def: "PipelineDefinition") -> DependencyGraph[str]:
+def generate_dep_graph(job_def: "GraphDefinition") -> DependencyGraph[str]:
     """Pipeline to dependency graph. It currently only supports top-level solids.
 
     Args:
-        pipeline (PipelineDefinition): The pipeline to execute.
+        pipeline (JobDefinition): The pipeline to execute.
 
     Returns:
         graph (Dict[str, Dict[str, Set[str]]]): the input and output dependency graph. e.g.
@@ -157,12 +129,12 @@ def generate_dep_graph(pipeline_def: "PipelineDefinition") -> DependencyGraph[st
             ```
     """
     dependency_structure = check.inst_param(
-        pipeline_def.dependency_structure, "dependency_structure", DependencyStructure
+        job_def.dependency_structure, "dependency_structure", DependencyStructure
     )
-    item_names = [i.name for i in pipeline_def.nodes]
+    item_names = [i.name for i in job_def.nodes]
 
     # defaultdict isn't appropriate because we also want to include items without dependencies
-    graph: Dict[Direction, Dict[str, MutableSet[str]]] = {"upstream": {}, "downstream": {}}
+    graph: dict[Direction, dict[str, MutableSet[str]]] = {"upstream": {}, "downstream": {}}
     for item_name in item_names:
         graph["upstream"][item_name] = set()
         upstream_dep = dependency_structure.input_to_upstream_outputs_for_node(item_name)
@@ -185,11 +157,11 @@ class Traverser(Generic[T_Hashable]):
 
     # `depth=None` is infinite depth
     def _fetch_items(
-        self, item_name: T_Hashable, depth: int, direction: Direction
+        self, item_names: Iterable[T_Hashable], depth: int, direction: Direction
     ) -> AbstractSet[T_Hashable]:
         dep_graph = self.graph[direction]
-        stack = deque([item_name])
-        result: Set[T_Hashable] = set()
+        stack = deque(item_names)
+        result: set[T_Hashable] = set()
         curr_depth = 0
         while stack:
             # stop when reach the given depth
@@ -199,7 +171,7 @@ class Traverser(Generic[T_Hashable]):
             while stack and curr_level_len > 0:
                 curr_item = stack.popleft()
                 curr_level_len -= 1
-                empty_set: Set[T_Hashable] = set()
+                empty_set: set[T_Hashable] = set()
                 for item in dep_graph.get(curr_item, empty_set):
                     if item not in result:
                         stack.append(item)
@@ -207,17 +179,25 @@ class Traverser(Generic[T_Hashable]):
             curr_depth += 1
         return result
 
+    def fetch_upstream_multiple(
+        self, item_names: Iterable[T_Hashable], depth: int
+    ) -> AbstractSet[T_Hashable]:
+        return self._fetch_items(item_names, depth, "upstream")
+
+    def fetch_downstream_multiple(
+        self, item_names: Iterable[T_Hashable], depth: int
+    ) -> AbstractSet[T_Hashable]:
+        return self._fetch_items(item_names, depth, "downstream")
+
     def fetch_upstream(self, item_name: T_Hashable, depth: int) -> AbstractSet[T_Hashable]:
-        # return a set of ancestors of the given item, up to the given depth
-        return self._fetch_items(item_name, depth, "upstream")
+        return self.fetch_upstream_multiple({item_name}, depth)
 
     def fetch_downstream(self, item_name: T_Hashable, depth: int) -> AbstractSet[T_Hashable]:
-        # return a set of descendants of the given item, down to the given depth
-        return self._fetch_items(item_name, depth, "downstream")
+        return self.fetch_downstream_multiple({item_name}, depth)
 
 
 def fetch_connected(
-    item: T_Hashable,
+    items: Iterable[T_Hashable],
     graph: DependencyGraph[T_Hashable],
     *,
     direction: Direction,
@@ -226,9 +206,9 @@ def fetch_connected(
     if depth is None:
         depth = MAX_NUM
     if direction == "downstream":
-        return Traverser(graph).fetch_downstream(item, depth)
+        return Traverser(graph).fetch_downstream_multiple(items, depth)
     elif direction == "upstream":
-        return Traverser(graph).fetch_upstream(item, depth)
+        return Traverser(graph).fetch_upstream_multiple(items, depth)
 
 
 def fetch_sinks(
@@ -238,7 +218,7 @@ def fetch_sinks(
     It can have other dependencies outside of the selection.
     """
     traverser = Traverser(graph)
-    sinks: Set[T_Hashable] = set()
+    sinks: set[T_Hashable] = set()
     for item in within_selection:
         downstream = traverser.fetch_downstream(item, depth=MAX_NUM) & within_selection
         if len(downstream) == 0 or downstream == {item}:
@@ -247,20 +227,20 @@ def fetch_sinks(
 
 
 def fetch_sources(
-    graph: DependencyGraph[T_Hashable], within_selection: AbstractSet[T_Hashable]
-) -> AbstractSet[T_Hashable]:
+    graph: "BaseAssetGraph[T_AssetNode]", within_selection: AbstractSet[AssetKey]
+) -> AbstractSet[AssetKey]:
     """A source is a node that has no upstream dependencies within the provided selection.
     It can have other dependencies outside of the selection.
     """
-    dp: Dict[T_Hashable, bool] = {}
+    dp: dict[AssetKey, bool] = {}
 
-    def has_upstream_within_selection(node: T_Hashable) -> bool:
-        if node not in dp:
-            dp[node] = any(
+    def has_upstream_within_selection(key: AssetKey) -> bool:
+        if key not in dp:
+            dp[key] = any(
                 parent_node in within_selection or has_upstream_within_selection(parent_node)
-                for parent_node in graph["upstream"].get(node, set()) - {node}
+                for parent_node in graph.get(key).parent_keys - {key}
             )
-        return dp[node]
+        return dp[key]
 
     return {node for node in within_selection if not has_upstream_within_selection(node)}
 
@@ -272,16 +252,20 @@ def fetch_connected_assets_definitions(
     *,
     direction: Direction,
     depth: Optional[int] = MAX_NUM,
-) -> FrozenSet["AssetsDefinition"]:
+) -> frozenset["AssetsDefinition"]:
     depth = MAX_NUM if depth is None else depth
     names = [asset_key.to_user_string() for asset_key in asset.keys]
-    connected_names = [
-        n for name in names for n in fetch_connected(name, graph, direction=direction, depth=depth)
-    ]
+    connected_names = list(fetch_connected(names, graph, direction=direction, depth=depth))
     return frozenset(name_to_definition_map[n] for n in connected_names)
 
 
-def parse_clause(clause: str) -> Optional[Tuple[int, str, int]]:
+class GraphSelectionClause(NamedTuple):
+    up_depth: int
+    item_name: str
+    down_depth: int
+
+
+def parse_clause(clause: str) -> Optional[GraphSelectionClause]:
     def _get_depth(part: str) -> int:
         if part == "":
             return 0
@@ -302,11 +286,11 @@ def parse_clause(clause: str) -> Optional[Tuple[int, str, int]]:
     up_depth = _get_depth(ancestor_part)
     down_depth = _get_depth(descendant_part)
 
-    return (up_depth, item_name, down_depth)
+    return GraphSelectionClause(up_depth, item_name, down_depth)
 
 
 def parse_items_from_selection(selection: Sequence[str]) -> Sequence[str]:
-    items: List[str] = []
+    items: list[str] = []
     for clause in selection:
         parts = parse_clause(clause)
         if parts is None:
@@ -343,7 +327,7 @@ def clause_to_subset(
     if item not in graph["upstream"]:
         return []
 
-    subset_list: List[T_Hashable] = []
+    subset_list: list[T_Hashable] = []
     traverser = Traverser(graph=graph)
     subset_list.append(item)
     # traverse graph to get up/downsteam items
@@ -353,65 +337,10 @@ def clause_to_subset(
     return subset_list
 
 
-class SelectionTreeLeaf:
-    """Marker for no further nesting selection needed."""
-
-
-SelectionTreeBranch: TypeAlias = Dict[str, "SelectionTree"]
-SelectionTree: TypeAlias = Union[SelectionTreeBranch, SelectionTreeLeaf]
-
-SELECTION_TREE_LEAF_NODE: Final = SelectionTreeLeaf()
-
-
-def convert_dot_separated_string_to_selection_tree(
-    tree: SelectionTreeBranch, splits: Sequence[str]
-) -> SelectionTreeBranch:
-    # For example:
-    # "subgraph.subsubgraph.return_one" => {"subgraph": {"subsubgraph": {"return_one":
-    # SELECTION_TREE_LEAF_NODE}}}
-    key = splits[0]
-    if len(splits) == 1:
-        tree[key] = SELECTION_TREE_LEAF_NODE
-    else:
-        curr_node = tree.get(key, {})
-        assert not isinstance(curr_node, SelectionTreeLeaf), "Invalid selection tree"
-        tree[key] = convert_dot_separated_string_to_selection_tree(curr_node, splits[1:])
-    return tree
-
-
-def parse_op_selection(
-    job_def: "JobDefinition", op_selection: Sequence[str]
-) -> SelectionTreeBranch:
-    """Parse  an op selection into a nested dictionary.
-
-    Examples:
-        ["subgraph.return_one", "subgraph.adder", "subgraph.add_one", "add_one"]
-        => {"subgraph": {{"return_one": LeafNodeSelection}, {"adder": LeafNodeSelection}, {"add_one": LeafNodeSelection}}, "add_one": LeafNodeSelection}
-
-        ["subgraph.subsubgraph.return_one"]
-        => {"subgraph": {"subsubgraph": {"return_one": LeafNodeSelection}}}
-
-        ["top_level_op_1+"]
-        => {"top_level_op_1": LeafNodeSelection, "top_level_op_2": LeafNodeSelection}
-    """
-    if any(["." in item for item in op_selection]):
-        resolved_op_selection_dict: Dict[str, Any] = {}
-        for item in op_selection:
-            convert_dot_separated_string_to_selection_tree(
-                resolved_op_selection_dict, splits=item.split(".")
-            )
-        return resolved_op_selection_dict
-
-    return {
-        top_level_op: SELECTION_TREE_LEAF_NODE
-        for top_level_op in parse_solid_selection(job_def, op_selection)
-    }
-
-
-def parse_solid_selection(
-    pipeline_def: "PipelineDefinition",
-    solid_selection: Sequence[str],
-) -> FrozenSet[str]:
+def parse_op_queries(
+    graph_def: "GraphDefinition",
+    op_queries: Sequence[str],
+) -> AbstractSet[str]:
     """Take pipeline definition and a list of solid selection queries (inlcuding names of solid
         invocations. See syntax examples below) and return a set of the qualified solid names.
 
@@ -430,42 +359,38 @@ def parse_solid_selection(
         ones.
 
     Args:
-        pipeline_def (PipelineDefinition): the pipeline to execute.
-        solid_selection (List[str]): a list of the solid selection queries (including single solid
+        pipeline_def (JobDefinition): the pipeline to execute.
+        op_queries (List[str]): a list of the solid selection queries (including single solid
             names) to execute.
 
     Returns:
         FrozenSet[str]: a frozenset of qualified deduplicated solid names, empty if no qualified
             subset selected.
     """
-    check.sequence_param(solid_selection, "solid_selection", of_type=str)
+    check.sequence_param(op_queries, "op_queries", of_type=str)
 
     # special case: select all
-    if len(solid_selection) == 1 and solid_selection[0] == "*":
-        return frozenset(pipeline_def.graph.node_names())
+    if len(op_queries) == 1 and op_queries[0] == "*":
+        return frozenset(graph_def.node_names())
 
-    graph = generate_dep_graph(pipeline_def)
-    solids_set: Set[str] = set()
+    graph = generate_dep_graph(graph_def)
+    node_names: set[str] = set()
 
     # loop over clauses
-    for clause in solid_selection:
+    for clause in op_queries:
         subset = clause_to_subset(graph, clause, lambda x: x)
         if len(subset) == 0:
             raise DagsterInvalidSubsetError(
-                "No qualified {node_type} to execute found for {selection_type}={requested}".format(
-                    requested=solid_selection,
-                    node_type="ops" if pipeline_def.is_job else "solids",
-                    selection_type="op_selection" if pipeline_def.is_job else "solid_selection",
-                )
+                f"No qualified ops to execute found for op_selection={op_queries}"
             )
-        solids_set.update(subset)
+        node_names.update(subset)
 
-    return frozenset(solids_set)
+    return node_names
 
 
 def parse_step_selection(
     step_deps: Mapping[str, AbstractSet[str]], step_selection: Sequence[str]
-) -> FrozenSet[str]:
+) -> frozenset[str]:
     """Take the dependency dictionary generated while building execution plan and a list of step key
      selection queries and return a set of the qualified step keys.
 
@@ -484,23 +409,21 @@ def parse_step_selection(
     check.sequence_param(step_selection, "step_selection", of_type=str)
     # reverse step_deps to get the downstream_deps
     # make sure we have all items as keys, including the ones without downstream dependencies
-    downstream_deps: Dict[str, Set[str]] = defaultdict(set, {k: set() for k in step_deps.keys()})
+    downstream_deps: dict[str, set[str]] = defaultdict(set, {k: set() for k in step_deps.keys()})
     for downstream_key, upstream_keys in step_deps.items():
         for step_key in upstream_keys:
             downstream_deps[step_key].add(downstream_key)
 
     # generate dep graph
     graph: DependencyGraph[str] = {"upstream": step_deps, "downstream": downstream_deps}
-    steps_set: Set[str] = set()
+    steps_set: set[str] = set()
 
     step_keys = parse_items_from_selection(step_selection)
     invalid_keys = [key for key in step_keys if key not in step_deps]
     if invalid_keys:
         raise DagsterExecutionStepNotFoundError(
-            (
-                f"Step selection refers to unknown step{'s' if len(invalid_keys)> 1 else ''}:"
-                f" {', '.join(invalid_keys)}"
-            ),
+            f"Step selection refers to unknown step{'s' if len(invalid_keys) > 1 else ''}:"
+            f" {', '.join(invalid_keys)}",
             step_keys=invalid_keys,
         )
 
@@ -509,9 +432,7 @@ def parse_step_selection(
         subset = clause_to_subset(graph, clause, lambda x: x)
         if len(subset) == 0:
             raise DagsterInvalidSubsetError(
-                "No qualified steps to execute found for step_selection={requested}".format(
-                    requested=step_selection
-                ),
+                f"No qualified steps to execute found for step_selection={step_selection}",
             )
         steps_set.update(subset)
 
@@ -520,7 +441,6 @@ def parse_step_selection(
 
 def parse_asset_selection(
     assets_defs: Sequence["AssetsDefinition"],
-    source_assets: Sequence["SourceAsset"],
     asset_selection: Sequence[str],
     raise_on_clause_has_no_matches: bool = True,
 ) -> AbstractSet[AssetKey]:
@@ -541,8 +461,8 @@ def parse_asset_selection(
     if len(asset_selection) == 1 and asset_selection[0] == "*":
         return {key for ad in assets_defs for key in ad.keys}
 
-    graph = generate_asset_dep_graph(assets_defs, source_assets)
-    assets_set: Set[AssetKey] = set()
+    graph = generate_asset_dep_graph(assets_defs)
+    assets_set: set[AssetKey] = set()
 
     # loop over clauses
     for clause in asset_selection:
