@@ -29,9 +29,13 @@ from dagster._core.execution.plan.state import KnownExecutionState
 from dagster._core.instance import DagsterInstance
 from dagster._core.instance.types import CachingDynamicPartitionsLoader
 from dagster._core.loader import LoadingContext
+from dagster._core.remote_origin import (
+    CodeLocationOrigin,
+    GrpcServerCodeLocationOrigin,
+    ManagedGrpcPythonEnvCodeLocationOrigin,
+)
 from dagster._core.remote_representation import (
     CodeLocation,
-    CodeLocationOrigin,
     GrpcServerCodeLocation,
     RemoteExecutionPlan,
     RemoteJob,
@@ -49,10 +53,6 @@ from dagster._core.remote_representation.grpc_server_state_subscriber import (
     LocationStateSubscriber,
 )
 from dagster._core.remote_representation.handle import InstigatorHandle
-from dagster._core.remote_representation.origin import (
-    GrpcServerCodeLocationOrigin,
-    ManagedGrpcPythonEnvCodeLocationOrigin,
-)
 from dagster._core.snap.dagster_types import DagsterTypeSnap
 from dagster._core.snap.mode import ResourceDefSnap
 from dagster._core.snap.node import GraphDefSnap, OpDefSnap
@@ -217,7 +217,9 @@ class BaseWorkspaceRequestContext(LoadingContext):
 
     @property
     def code_location_names(self) -> Sequence[str]:
-        return list(self.get_code_location_entries())
+        # For some WorkspaceRequestContext subclasses, the CodeLocationEntry is more expensive
+        # than the CodeLocationStatusEntry, so use the latter for a faster check.
+        return [status_entry.location_name for status_entry in self.get_code_location_statuses()]
 
     def code_location_errors(self) -> Sequence[SerializableErrorInfo]:
         return [
