@@ -1,5 +1,6 @@
+from collections.abc import Mapping
 from enum import Enum, unique
-from typing import Dict, Mapping, NamedTuple, Optional
+from typing import NamedTuple, Optional
 
 
 @unique
@@ -15,15 +16,18 @@ class Permissions(str, Enum):
     RELOAD_REPOSITORY_LOCATION = "reload_repository_location"
     RELOAD_WORKSPACE = "reload_workspace"
     WIPE_ASSETS = "wipe_assets"
+    REPORT_RUNLESS_ASSET_EVENTS = "report_runless_asset_events"
     LAUNCH_PARTITION_BACKFILL = "launch_partition_backfill"
     CANCEL_PARTITION_BACKFILL = "cancel_partition_backfill"
     EDIT_DYNAMIC_PARTITIONS = "edit_dynamic_partitions"
+    TOGGLE_AUTO_MATERIALIZE = "toggle_auto_materialize"
+    EDIT_CONCURRENCY_LIMIT = "edit_concurrency_limit"
 
     def __str__(self) -> str:
         return str.__str__(self)
 
 
-VIEWER_PERMISSIONS: Dict[str, bool] = {
+VIEWER_PERMISSIONS: dict[str, bool] = {
     Permissions.LAUNCH_PIPELINE_EXECUTION: False,
     Permissions.LAUNCH_PIPELINE_REEXECUTION: False,
     Permissions.START_SCHEDULE: False,
@@ -35,12 +39,15 @@ VIEWER_PERMISSIONS: Dict[str, bool] = {
     Permissions.RELOAD_REPOSITORY_LOCATION: False,
     Permissions.RELOAD_WORKSPACE: False,
     Permissions.WIPE_ASSETS: False,
+    Permissions.REPORT_RUNLESS_ASSET_EVENTS: False,
     Permissions.LAUNCH_PARTITION_BACKFILL: False,
     Permissions.CANCEL_PARTITION_BACKFILL: False,
     Permissions.EDIT_DYNAMIC_PARTITIONS: False,
+    Permissions.TOGGLE_AUTO_MATERIALIZE: False,
+    Permissions.EDIT_CONCURRENCY_LIMIT: False,
 }
 
-EDITOR_PERMISSIONS: Dict[str, bool] = {
+EDITOR_PERMISSIONS: dict[str, bool] = {
     Permissions.LAUNCH_PIPELINE_EXECUTION: True,
     Permissions.LAUNCH_PIPELINE_REEXECUTION: True,
     Permissions.START_SCHEDULE: True,
@@ -52,9 +59,12 @@ EDITOR_PERMISSIONS: Dict[str, bool] = {
     Permissions.RELOAD_REPOSITORY_LOCATION: True,
     Permissions.RELOAD_WORKSPACE: True,
     Permissions.WIPE_ASSETS: True,
+    Permissions.REPORT_RUNLESS_ASSET_EVENTS: True,
     Permissions.LAUNCH_PARTITION_BACKFILL: True,
     Permissions.CANCEL_PARTITION_BACKFILL: True,
     Permissions.EDIT_DYNAMIC_PARTITIONS: True,
+    Permissions.TOGGLE_AUTO_MATERIALIZE: True,
+    Permissions.EDIT_CONCURRENCY_LIMIT: True,
 }
 
 LOCATION_SCOPED_PERMISSIONS = {
@@ -70,6 +80,8 @@ LOCATION_SCOPED_PERMISSIONS = {
     Permissions.LAUNCH_PARTITION_BACKFILL,
     Permissions.CANCEL_PARTITION_BACKFILL,
     Permissions.EDIT_DYNAMIC_PARTITIONS,
+    Permissions.REPORT_RUNLESS_ASSET_EVENTS,
+    Permissions.WIPE_ASSETS,
 }
 
 
