@@ -8,7 +8,8 @@ from dagster._utils.error import SerializableErrorInfo
 
 if TYPE_CHECKING:
     from dagster._core.definitions.assets.graph.remote_asset_graph import RemoteWorkspaceAssetGraph
-    from dagster._core.remote_representation import CodeLocation, CodeLocationOrigin
+    from dagster._core.remote_origin import CodeLocationOrigin
+    from dagster._core.remote_representation import CodeLocation
 
 
 # For locations that are loaded asynchronously
@@ -19,7 +20,7 @@ class CodeLocationLoadStatus(Enum):
 
 @record
 class CodeLocationEntry:
-    origin: Annotated["CodeLocationOrigin", ImportFrom("dagster._core.remote_representation")]
+    origin: Annotated["CodeLocationOrigin", ImportFrom("dagster._core.remote_origin")]
     code_location: Optional[
         Annotated["CodeLocation", ImportFrom("dagster._core.remote_representation")]
     ]
