@@ -1,19 +1,25 @@
 import logging
 import os
-from typing import Callable, Dict, Iterator, List, Mapping, Optional, Sequence
+import sys
+from collections.abc import Iterator, Mapping, Sequence
+from typing import TYPE_CHECKING, Callable, Optional
 
 import dagster._check as check
 from dagster import DagsterEventType
-from dagster._core.events.log import EventLogEntry
 from dagster._core.instance import DagsterInstance
-from dagster._core.storage.pipeline_run import RunRecord, RunsFilter
+from dagster._core.storage.dagster_run import RunRecord, RunsFilter
 from dagster._core.workspace.context import IWorkspaceProcessContext
+from dagster._daemon.auto_run_reexecution.auto_run_reexecution import (
+    consume_new_runs_for_automatic_reexecution,
+)
+from dagster._daemon.daemon import IntervalDaemon
+from dagster._daemon.utils import DaemonErrorCapture
 
-from ..daemon import IntervalDaemon
-from .auto_run_reexecution import consume_new_runs_for_automatic_reexecution
+if TYPE_CHECKING:
+    from dagster._core.events.log import EventLogEntry
 
-_INTERVAL_SECONDS = int(os.environ.get("DAGSTER_EVENT_LOG_CONSUMER_DAEMON_INTERVAL_SECONDS", 5))
-_EVENT_LOG_FETCH_LIMIT = int(os.environ.get("DAGSTER_EVENT_LOG_CONSUMER_DAEMON_FETCH_LIMIT", 500))
+_INTERVAL_SECONDS = int(os.environ.get("DAGSTER_EVENT_LOG_CONSUMER_DAEMON_INTERVAL_SECONDS", "5"))
+_EVENT_LOG_FETCH_LIMIT = int(os.environ.get("DAGSTER_EVENT_LOG_CONSUMER_DAEMON_FETCH_LIMIT", "500"))
 
 DAGSTER_EVENT_TYPES = [DagsterEventType.RUN_FAILURE, DagsterEventType.RUN_SUCCESS]
 
@@ -24,7 +30,7 @@ class EventLogConsumerDaemon(IntervalDaemon):
         interval_seconds: int = _INTERVAL_SECONDS,
         event_log_fetch_limit: int = _EVENT_LOG_FETCH_LIMIT,
     ):
-        super(EventLogConsumerDaemon, self).__init__(interval_seconds=interval_seconds)
+        super().__init__(interval_seconds=interval_seconds)
         self._event_log_fetch_limit = event_log_fetch_limit
 
     @classmethod
@@ -34,9 +40,10 @@ class EventLogConsumerDaemon(IntervalDaemon):
     @property
     def handle_updated_runs_fns(
         self,
-    ) -> Sequence[Callable[[IWorkspaceProcessContext, Sequence[RunRecord]], Iterator]]:
-        """List of functions that will be called with the list of run records that have new events.
-        """
+    ) -> Sequence[
+        Callable[[IWorkspaceProcessContext, Sequence[RunRecord], logging.Logger], Iterator]
+    ]:
+        """List of functions that will be called with the list of run records that have new events."""
         return [consume_new_runs_for_automatic_reexecution]
 
     def run_iteration(self, workspace_process_context: IWorkspaceProcessContext):
@@ -47,8 +54,8 @@ class EventLogConsumerDaemon(IntervalDaemon):
         # Get the current greatest event id before we query for the specific event types
         overall_max_event_id = instance.event_log_storage.get_maximum_record_id()
 
-        events: List[EventLogEntry] = []
-        new_cursors: Dict[
+        events: list[EventLogEntry] = []
+        new_cursors: dict[
             DagsterEventType, int
         ] = {}  # keep these in memory until we handle the events
         for event_type in DAGSTER_EVENT_TYPES:
@@ -82,12 +89,12 @@ class EventLogConsumerDaemon(IntervalDaemon):
             # call each handler with the list of runs that have events
             for fn in self.handle_updated_runs_fns:
                 try:
-                    yield from fn(workspace_process_context, run_records)
+                    yield from fn(workspace_process_context, run_records, self._logger)
                 except Exception:
-                    self._logger.exception(
-                        "Error calling event event log consumer handler: {handler_fn}".format(
-                            handler_fn=fn.__name__,
-                        )
+                    DaemonErrorCapture.process_exception(
+                        sys.exc_info(),
+                        logger=self._logger,
+                        log_message=f"Error calling event event log consumer handler: {fn.__name__}",
                     )
 
         # persist cursors now that we've processed all the events through the handlers
@@ -102,16 +109,16 @@ def _create_cursor_key(event_type: DagsterEventType) -> str:
 
 def _fetch_persisted_cursors(
     instance: DagsterInstance, event_types: Sequence[DagsterEventType], logger: logging.Logger
-) -> Dict[DagsterEventType, Optional[int]]:
+) -> dict[DagsterEventType, Optional[int]]:
     check.inst_param(instance, "instance", DagsterInstance)
     check.sequence_param(event_types, "event_types", of_type=DagsterEventType)
 
     # get the persisted cursor for each event type
-    persisted_cursors = instance.run_storage.kvs_get(
+    persisted_cursors = instance.daemon_cursor_storage.get_cursor_values(
         {_create_cursor_key(event_type) for event_type in event_types}
     )
 
-    fetched_cursors: Dict[DagsterEventType, Optional[int]] = {}
+    fetched_cursors: dict[DagsterEventType, Optional[int]] = {}
     for event_type in event_types:
         raw_cursor_value = persisted_cursors.get(_create_cursor_key(event_type))
 
@@ -134,7 +141,7 @@ def _persist_cursors(instance: DagsterInstance, cursors: Mapping[DagsterEventTyp
     check.mapping_param(cursors, "cursors", key_type=DagsterEventType, value_type=int)
 
     if cursors:
-        instance.run_storage.kvs_set(
+        instance.daemon_cursor_storage.set_cursor_values(
             {
                 _create_cursor_key(event_type): str(cursor_value)
                 for event_type, cursor_value in cursors.items()
@@ -185,9 +192,7 @@ def get_new_cursor(
 
     check.invariant(
         max_new_event_id > persisted_cursor,
-        "The new cursor {} should be greater than the previous {}".format(
-            max_new_event_id, persisted_cursor
-        ),
+        f"The new cursor {max_new_event_id} should be greater than the previous {persisted_cursor}",
     )
 
     num_new_events = len(new_event_ids)
