@@ -4,18 +4,19 @@ import sys
 import threading
 import time
 import uuid
+from collections.abc import Iterable, Iterator, Mapping, Sequence
 from contextlib import AbstractContextManager, ExitStack, contextmanager
 from types import TracebackType
-from typing import Callable, Dict, Iterable, Iterator, Mapping, Optional, Sequence, Type
+from typing import Callable, Optional
 
-import pendulum
 from typing_extensions import Self
 
 import dagster._check as check
-from dagster._core.host_representation.grpc_server_registry import GrpcServerRegistry
 from dagster._core.instance import DagsterInstance
+from dagster._core.remote_representation.grpc_server_registry import GrpcServerRegistry
 from dagster._core.workspace.context import IWorkspaceProcessContext, WorkspaceProcessContext
 from dagster._core.workspace.load_target import WorkspaceLoadTarget
+from dagster._daemon.asset_daemon import AssetDaemon
 from dagster._daemon.auto_run_reexecution.event_log_consumer import EventLogConsumerDaemon
 from dagster._daemon.daemon import (
     BackfillDaemon,
@@ -24,13 +25,16 @@ from dagster._daemon.daemon import (
     SchedulerDaemon,
     SensorDaemon,
 )
+from dagster._daemon.freshness import FreshnessDaemon
 from dagster._daemon.run_coordinator.queued_run_coordinator_daemon import QueuedRunCoordinatorDaemon
 from dagster._daemon.types import DaemonHeartbeat, DaemonStatus
+from dagster._grpc.constants import INCREASE_TIMEOUT_DAGSTER_YAML_MSG, GrpcServerCommand
+from dagster._time import get_current_datetime, get_current_timestamp
 from dagster._utils.interrupts import raise_interrupts_as
 from dagster._utils.log import configure_loggers
 
 # How long beyond the expected heartbeat will the daemon be considered healthy
-DEFAULT_DAEMON_HEARTBEAT_TOLERANCE_SECONDS = 300
+DEFAULT_DAEMON_HEARTBEAT_TOLERANCE_SECONDS = 1800
 
 # Default interval at which daemons run
 DEFAULT_DAEMON_INTERVAL_SECONDS = 30
@@ -43,12 +47,17 @@ DEFAULT_DAEMON_ERROR_INTERVAL_SECONDS = 300
 
 THREAD_CHECK_INTERVAL = 5
 
-HEARTBEAT_CHECK_INTERVAL = 15
+HEARTBEAT_CHECK_INTERVAL = 60
 
 RELOAD_WORKSPACE_INTERVAL = 60
 
-DAEMON_GRPC_SERVER_RELOAD_INTERVAL = 60
-DAEMON_GRPC_SERVER_HEARTBEAT_TTL = 120
+# Number of seconds the workspace can fail to refresh before restarting the daemon.
+DEFAULT_WORKSPACE_FRESHNESS_TOLERANCE = 300
+
+# Amount of time that a local code server spun up by the daemon will keep running
+# after it is no longer receiving any heartbeat pings - for this duration there may be
+# multiple code server processes running
+DAEMON_GRPC_SERVER_HEARTBEAT_TTL = 20
 
 
 def _sorted_quoted(strings: Iterable[str]) -> str:
@@ -66,11 +75,13 @@ def create_daemon_grpc_server_registry(
     instance: DagsterInstance, code_server_log_level: str = "INFO"
 ) -> GrpcServerRegistry:
     return GrpcServerRegistry(
-        instance=instance,
-        reload_interval=DAEMON_GRPC_SERVER_RELOAD_INTERVAL,
+        instance_ref=instance.get_ref(),
+        server_command=GrpcServerCommand.API_GRPC,
         heartbeat_ttl=DAEMON_GRPC_SERVER_HEARTBEAT_TTL,
         startup_timeout=instance.code_server_process_startup_timeout,
+        additional_timeout_msg=INCREASE_TIMEOUT_DAGSTER_YAML_MSG,
         log_level=code_server_log_level,
+        wait_for_processes_on_shutdown=instance.wait_for_local_code_server_processes_on_shutdown,
     )
 
 
@@ -85,6 +96,8 @@ def daemon_controller_from_instance(
     ] = create_daemons_from_instance,
     error_interval_seconds: int = DEFAULT_DAEMON_ERROR_INTERVAL_SECONDS,
     code_server_log_level: str = "info",
+    log_level: str = "info",
+    log_format: str = "colored",
 ) -> Iterator["DagsterDaemonController"]:
     check.inst_param(instance, "instance", DagsterInstance)
     check.inst_param(workspace_load_target, "workspace_load_target", WorkspaceLoadTarget)
@@ -101,6 +114,9 @@ def daemon_controller_from_instance(
                 grpc_server_registry=grpc_server_registry,
             )
         )
+
+        configure_loggers(handler="default", formatter=log_format, log_level=log_level.upper())
+
         controller = stack.enter_context(
             DagsterDaemonController(
                 workspace_process_context,
@@ -108,6 +124,7 @@ def daemon_controller_from_instance(
                 heartbeat_interval_seconds=heartbeat_interval_seconds,
                 heartbeat_tolerance_seconds=heartbeat_tolerance_seconds,
                 error_interval_seconds=error_interval_seconds,
+                grpc_server_registry=grpc_server_registry,
             )
         )
 
@@ -116,25 +133,26 @@ def daemon_controller_from_instance(
 
 class DagsterDaemonController(AbstractContextManager):
     _daemon_uuid: str
-    _daemons: Dict[str, DagsterDaemon]
-    _daemon_threads: Dict[str, threading.Thread]
+    _daemons: dict[str, DagsterDaemon]
+    _grpc_server_registry: Optional[GrpcServerRegistry]
+    _daemon_threads: dict[str, threading.Thread]
     _workspace_process_context: IWorkspaceProcessContext
     _instance: DagsterInstance
     _heartbeat_interval_seconds: float
     _heartbeat_tolerance_seconds: float
     _daemon_shutdown_event: threading.Event
     _logger: logging.Logger
-    _last_healthy_heartbeat_times: Dict[str, float]
+    _last_healthy_heartbeat_times: dict[str, float]
     _start_time: datetime.datetime
 
     def __init__(
         self,
         workspace_process_context: IWorkspaceProcessContext,
         daemons: Sequence[DagsterDaemon],
+        grpc_server_registry: Optional[GrpcServerRegistry] = None,
         heartbeat_interval_seconds: float = DEFAULT_HEARTBEAT_INTERVAL_SECONDS,
         heartbeat_tolerance_seconds: float = DEFAULT_DAEMON_HEARTBEAT_TOLERANCE_SECONDS,
         error_interval_seconds: int = DEFAULT_DAEMON_ERROR_INTERVAL_SECONDS,
-        handler: str = "default",
     ):
         self._daemon_uuid = str(uuid.uuid4())
 
@@ -153,13 +171,13 @@ class DagsterDaemonController(AbstractContextManager):
             heartbeat_tolerance_seconds, "heartbeat_tolerance_seconds"
         )
 
+        self._grpc_server_registry = grpc_server_registry
+
         if not self._daemons:
             raise Exception("No daemons configured on the DagsterInstance")
 
         self._daemon_shutdown_event = threading.Event()
 
-        configure_loggers(handler=handler)
-
         self._logger = logging.getLogger("dagster.daemon")
         self._logger.info(
             "Instance is configured with the following daemons: %s",
@@ -184,7 +202,7 @@ class DagsterDaemonController(AbstractContextManager):
             self._last_healthy_heartbeat_times[daemon_type] = time.time()
             self._daemon_threads[daemon_type].start()
 
-        self._start_time = pendulum.now("UTC")
+        self._start_time = get_current_datetime()
 
     def __enter__(self) -> Self:
         return self
@@ -242,18 +260,36 @@ class DagsterDaemonController(AbstractContextManager):
             raise Exception("Stopped dagster-daemon process due to threads no longer running")
 
     def check_daemon_heartbeats(self) -> None:
-        failed_daemons = [
+        no_heartbeat_daemons = [
             daemon_type
             for daemon_type, is_daemon_healthy in self._daemon_heartbeat_health().items()
             if not is_daemon_healthy
         ]
 
-        if failed_daemons:
-            self._logger.error(
-                "Stopping dagster-daemon process since the following threads are no longer sending"
-                f" heartbeats: {failed_daemons}"
+        if no_heartbeat_daemons:
+            self._logger.warning(
+                "The following threads have not sent heartbeats in more than"
+                f" {self._heartbeat_tolerance_seconds} seconds: {no_heartbeat_daemons}."
+                " They may be running more slowly than expected or hanging."
             )
-            raise Exception("Stopped dagster-daemon process due to thread heartbeat failure")
+
+    def check_workspace_freshness(self, last_workspace_update_time: float) -> float:
+        nowish = get_current_timestamp()
+        try:
+            if (nowish - last_workspace_update_time) > RELOAD_WORKSPACE_INTERVAL:
+                if self._grpc_server_registry:
+                    self._grpc_server_registry.clear_all_grpc_endpoints()
+                self._workspace_process_context.refresh_workspace()
+                return get_current_timestamp()
+        except Exception:
+            if (nowish - last_workspace_update_time) > DEFAULT_WORKSPACE_FRESHNESS_TOLERANCE:
+                self._logger.exception("Daemon controller surpassed workspace freshness tolerance.")
+                raise
+            else:
+                self._logger.exception(
+                    "Daemon controller failed to refresh workspace. Still within freshness tolerance."
+                )
+        return last_workspace_update_time
 
     def check_daemon_loop(self) -> None:
         start_time = time.time()
@@ -265,16 +301,16 @@ class DagsterDaemonController(AbstractContextManager):
                 self.check_daemon_threads()
 
                 # periodically refresh the shared workspace context
-                if (time.time() - last_workspace_update_time) > RELOAD_WORKSPACE_INTERVAL:
-                    self._workspace_process_context.reload_workspace()
-                    last_workspace_update_time = time.time()
+                last_workspace_update_time = self.check_workspace_freshness(
+                    last_workspace_update_time
+                )
 
                 if self._instance.daemon_skip_heartbeats_without_errors:
                     # If we're skipping heartbeats without errors, we just check the threads.
                     # If there's no errors, the daemons won't be writing heartbeats.
                     continue
 
-                now = time.time()
+                now = get_current_timestamp()
                 # Give the daemon enough time to send an initial heartbeat before checking
                 if (
                     (now - start_time) < 2 * self._heartbeat_interval_seconds
@@ -283,11 +319,11 @@ class DagsterDaemonController(AbstractContextManager):
                     continue
 
                 self.check_daemon_heartbeats()
-                last_heartbeat_check_time = time.time()
+                last_heartbeat_check_time = get_current_timestamp()
 
-    def __exit__(
+    def __exit__(  # pyright: ignore[reportIncompatibleMethodOverride]
         self,
-        exception_type: Type[BaseException],
+        exception_type: type[BaseException],
         exception_value: Exception,
         traceback: TracebackType,
     ) -> None:
@@ -323,17 +359,28 @@ def create_daemon_of_type(daemon_type: str, instance: DagsterInstance) -> Dagste
     if daemon_type == SchedulerDaemon.daemon_type():
         return SchedulerDaemon()
     elif daemon_type == SensorDaemon.daemon_type():
-        return SensorDaemon()
+        return SensorDaemon(settings=instance.get_sensor_settings())
     elif daemon_type == QueuedRunCoordinatorDaemon.daemon_type():
         return QueuedRunCoordinatorDaemon(
             interval_seconds=instance.run_coordinator.dequeue_interval_seconds  # type: ignore  # (??)
         )
     elif daemon_type == BackfillDaemon.daemon_type():
-        return BackfillDaemon(interval_seconds=DEFAULT_DAEMON_INTERVAL_SECONDS)
+        return BackfillDaemon(settings=instance.get_backfill_settings())
     elif daemon_type == MonitoringDaemon.daemon_type():
         return MonitoringDaemon(interval_seconds=instance.run_monitoring_poll_interval_seconds)
     elif daemon_type == EventLogConsumerDaemon.daemon_type():
         return EventLogConsumerDaemon()
+    elif daemon_type == AssetDaemon.daemon_type():
+        return AssetDaemon(
+            settings=instance.get_auto_materialize_settings(),
+            pre_sensor_interval_seconds=(
+                instance.auto_materialize_minimum_interval_seconds
+                if instance.auto_materialize_minimum_interval_seconds is not None
+                else DEFAULT_DAEMON_INTERVAL_SECONDS
+            ),
+        )
+    elif daemon_type == FreshnessDaemon.daemon_type():
+        return FreshnessDaemon()
     else:
         raise Exception(f"Unexpected daemon type {daemon_type}")
 
@@ -362,8 +409,7 @@ def all_daemons_live(
     heartbeat_interval_seconds: float = DEFAULT_HEARTBEAT_INTERVAL_SECONDS,
     heartbeat_tolerance_seconds: float = DEFAULT_DAEMON_HEARTBEAT_TOLERANCE_SECONDS,
 ) -> bool:
-    """True if all required daemons have had a recent heartbeat, regardless of if it contained errors.
-    """
+    """True if all required daemons have had a recent heartbeat, regardless of if it contained errors."""
     statuses_by_type = get_daemon_statuses(
         instance,
         daemon_types=instance.get_required_daemon_types(),
@@ -385,10 +431,10 @@ def get_daemon_statuses(
     heartbeat_tolerance_seconds: float = DEFAULT_DAEMON_HEARTBEAT_TOLERANCE_SECONDS,
 ) -> Mapping[str, DaemonStatus]:
     curr_time_seconds = check.opt_float_param(
-        curr_time_seconds, "curr_time_seconds", default=pendulum.now("UTC").float_timestamp
+        curr_time_seconds, "curr_time_seconds", default=get_current_timestamp()
     )
 
-    daemon_statuses_by_type: Dict[str, DaemonStatus] = {}
+    daemon_statuses_by_type: dict[str, DaemonStatus] = {}
     heartbeats = instance.get_daemon_heartbeats()
 
     for daemon_type in daemon_types:
@@ -410,7 +456,7 @@ def get_daemon_statuses(
                 maximum_tolerated_time = (
                     hearbeat_timestamp + heartbeat_interval_seconds + heartbeat_tolerance_seconds
                 )
-                healthy = curr_time_seconds <= maximum_tolerated_time  # type: ignore  # (possible none)
+                healthy = curr_time_seconds <= maximum_tolerated_time
 
                 if not ignore_errors and latest_heartbeat.errors:
                     healthy = False
@@ -426,8 +472,8 @@ def get_daemon_statuses(
 
 
 def debug_daemon_heartbeats(instance: DagsterInstance) -> None:
-    daemon = SensorDaemon()
-    timestamp = pendulum.now("UTC").float_timestamp
+    daemon = SensorDaemon(settings=instance.get_sensor_settings())
+    timestamp = get_current_timestamp()
     instance.add_daemon_heartbeat(DaemonHeartbeat(timestamp, daemon.daemon_type(), None, None))
     returned_timestamp = instance.get_daemon_heartbeats()[daemon.daemon_type()].timestamp
     print(f"Written timestamp: {timestamp}\nRead timestamp: {returned_timestamp}")  # noqa: T201
