@@ -4,14 +4,23 @@ import tarfile
 from io import BytesIO
 
 import click
-import requests
+from dagster_shared.scaffold import should_skip_scaffolded_file
 
-from .generate import _should_skip_file
-
-# Currently we only download from 'master' branch
-DEFAULT_GITHUB_URL = "https://codeload.github.com/dagster-io/dagster/tar.gz/master"
 # Examples aren't that can't be downloaded from the dagster project CLI
-EXAMPLES_TO_IGNORE = ["docs_snippets", "experimental"]
+EXAMPLES_TO_IGNORE = [
+    "components_yaml_checks_dsl",
+    "deploy_k8s_beta",
+    "docs_beta_snippets",
+    "docs_projects",
+    "docs_snippets",
+    "experimental",
+    "use_case_repository",
+    "airlift-mwaa-example",
+    "starlift-demo",
+    "pyproject.toml",
+    "README.md",
+    "temp_pins.txt",
+]
 # Hardcoded list of available examples. The list is tested against the examples folder in this mono
 # repo to make sure it's up-to-date.
 AVAILABLE_EXAMPLES = [
@@ -21,27 +30,52 @@ AVAILABLE_EXAMPLES = [
     "assets_pandas_pyspark",
     "assets_pandas_type_metadata",
     "assets_smoke_test",
-    "quickstart_aws",
-    "quickstart_etl",
-    "quickstart_gcp",
-    "quickstart_snowflake",
-    "tutorial_dbt_dagster",
-    "tutorial_notebook_assets",
     "deploy_docker",
     "deploy_ecs",
     "deploy_k8s",
     "development_to_production",
     "feature_graph_backed_assets",
+    "getting_started_etl_tutorial",
+    "google_drive_factory",
+    "oss-metadata-to-plus",
+    "project_analytics",
+    "project_dagster_university_start",
+    "project_du_dbt_starter",
     "project_fully_featured",
-    "with_airflow",
+    "quickstart_aws",
+    "quickstart_etl",
+    "quickstart_gcp",
+    "quickstart_snowflake",
+    "tutorial",
+    "tutorial_notebook_assets",
     "with_great_expectations",
+    "with_openai",
     "with_pyspark",
     "with_pyspark_emr",
     "with_wandb",
+    "airlift-federation-tutorial",
+    "airlift-migration-tutorial",
 ]
 
 
-def download_example_from_github(path: str, example: str):
+def _get_target_for_version(version: str) -> str:
+    if version == "1!0+dev":
+        target = "master"
+    else:
+        target = version
+    return target
+
+
+def _get_url_for_version(version: str) -> str:
+    return (
+        f"https://codeload.github.com/dagster-io/dagster/tar.gz/{_get_target_for_version(version)}"
+    )
+
+
+def download_example_from_github(path: str, example: str, version: str):
+    # defer for import performance
+    import requests
+
     if example not in AVAILABLE_EXAMPLES:
         click.echo(
             click.style(
@@ -52,12 +86,10 @@ def download_example_from_github(path: str, example: str):
         )
         sys.exit(1)
 
-    path_to_new_project = os.path.normpath(path)
-    path_to_selected_example = f"dagster-master/examples/{example}"
-
+    path_to_selected_example = f"dagster-{_get_target_for_version(version)}/examples/{example}/"
     click.echo(f"Downloading example '{example}'. This may take a while.")
 
-    response = requests.get(DEFAULT_GITHUB_URL, stream=True)
+    response = requests.get(_get_url_for_version(version), stream=True)
     with tarfile.open(fileobj=BytesIO(response.raw.read()), mode="r:gz") as tar_file:
         # Extract the selected example folder to destination
         subdir_and_files = [
@@ -66,10 +98,10 @@ def download_example_from_github(path: str, example: str):
             if tarinfo.name.startswith(path_to_selected_example)
         ]
         for member in subdir_and_files:
-            if _should_skip_file(member.name):
+            if should_skip_scaffolded_file(member.name):
                 continue
 
-            dest = member.name.replace(path_to_selected_example, path_to_new_project)
+            dest = member.name.replace(path_to_selected_example, path)
 
             if member.isdir():
                 os.mkdir(dest)
