@@ -1,8 +1,13 @@
 import pdb
 import sys
+import types
+from typing import Optional
+
+from dagster._annotations import public
 
 
 # From https://stackoverflow.com/questions/4716533/how-to-attach-debugger-to-a-python-subproccess
+@public
 class ForkedPdb(pdb.Pdb):
     """A pdb subclass that may be used from a forked multiprocessing child.
 
@@ -20,11 +25,15 @@ class ForkedPdb(pdb.Pdb):
 
             # some other complicated stuff
 
-    You can initiate pipeline execution via dagit and use the pdb debugger to examine/step through
+    You can initiate pipeline execution via the webserver and use the pdb debugger to examine/step through
     execution at the breakpoint.
     """
 
-    def interaction(self, frame, traceback):
+    def interaction(
+        self,
+        frame: Optional[types.FrameType],
+        traceback: Optional[types.TracebackType],
+    ):
         _stdin = sys.stdin
         try:
             sys.stdin = open("/dev/stdin", encoding="utf8")
