@@ -13,7 +13,7 @@ from dagster._core.definitions.declarative_automation.serialized_objects import
 from dagster._core.definitions.events import AssetKey
 from dagster._core.definitions.metadata import MetadataValue
 from dagster._core.definitions.partitions.definition import StaticPartitionsDefinition
-from dagster._core.remote_representation import (
+from dagster._core.remote_origin import (
     ManagedGrpcPythonEnvCodeLocationOrigin,
     RemoteRepositoryOrigin,
 )
